# -- (c) Copyright 2010 - 2011 Xilinx, Inc. All rights reserved.
# --
# -- This file contains confidential and proprietary information
# -- of Xilinx, Inc. and is protected under U.S. and 
# -- international copyright and other intellectual property
# -- laws.
# --
# -- DISCLAIMER
# -- This disclaimer is not a license and does not grant any
# -- rights to the materials distributed herewith. Except as
# -- otherwise provided in a valid license issued to you by
# -- Xilinx, and to the maximum extent permitted by applicable
# -- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
# -- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
# -- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
# -- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
# -- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
# -- (2) Xilinx shall not be liable (whether in contract or tort,
# -- including negligence, or under any other theory of
# -- liability) for any loss or damage of any kind or nature
# -- related to, arising under or in connection with these
# -- materials, including for any direct, or any indirect,
# -- special, incidental, or consequential loss or damage
# -- (including loss of data, profits, goodwill, or any type of
# -- loss or damage suffered as a result of any action brought
# -- by a third party) even if such damage or loss was
# -- reasonably foreseeable or Xilinx had been advised of the
# -- possibility of the same.
# --
# -- CRITICAL APPLICATIONS
# -- Xilinx products are not designed or intended to be fail-
# -- safe, or for use in any application requiring fail-safe
# -- performance, such as life-support or safety devices or
# -- systems, Class III medical devices, nuclear facilities,
# -- applications related to the deployment of airbags, or any
# -- other applications that could lead to death, personal
# -- injury, or severe property or environmental damage
# -- (individually and collectively, "Critical
# -- Applications"). Customer assumes the sole risk and
# -- liability of any use of Xilinx products in Critical
# -- Applications, subject only to applicable laws and
# -- regulations governing limitations on product liability.
# --
# -- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
# -- PART OF THIS FILE AT ALL TIMES.
# --------------------------------------------------------------------

#
# Common procs
# Output warning message
proc warning { message } {
   puts -nonewline "WARNING ****************************************************************"
   set index     0
   set linefirst 0
   set linelast  0
   set length    [string length $message]
   while {$index < $length} {
     if {[string index $message $index] == " "} {
       set linelast $index
     }
     if {[expr $index - $linefirst] >= 64} {
       puts -nonewline "WARNING [string range $message $linefirst $linelast]"
       set linefirst [expr $linelast + 1]
     }
     incr index
     if {($index == $length) && ($linefirst < $length)} {
       puts "WARNING [string range $message $linefirst end]"
     }
   }
}


#
# Update value of C_BASEFAMILY

proc iplevel_update_basefamily { param_handle } { 
    set mhsinst     [xget_hw_parent_handle   $param_handle]
    set family      [xget_hw_parameter_value $mhsinst "C_FAMILY"]
    if {[string match -nocase "*virtex7*" $family]} {return "virtex7"}
    if {[string match -nocase "*kintex7*" $family]} {return "kintex7"}
    if {[string match -nocase "*artix7*" $family]} {return "artix7"}
    if {[string match -nocase "*zynq*" $family]} {return "zynq"}
    if {[string match -nocase "*virtex6*" $family]} {return "virtex6"}
    if {[string match -nocase "*spartan6*" $family]} {return "spartan6"}
    puts "\nINTERNAL ERROR: C_FAMILY value $family not supported by axi_interconnect."
    puts "All data-path FIFOs (if any) are implemented as LUT RAM."
    return "rtl"
}

#
# Update value of C_RANGE_CHECK
#   This proc is called only if C_RANGE_CHECK is not set in MHS.
#
proc syslevel_update_range_check {param_handle} {
    set ON 1
    set OFF 0
    set MAX_NUM_SLOTS 16
    set MAX_NUM_RANGES 16
    set AXILITE "2"
    set ZERO_B1 "0b[string repeat "0" $MAX_NUM_SLOTS]"
    set mhsinst [xget_hw_parent_handle $param_handle]
    set instname [xget_hw_parameter_value $mhsinst "INSTANCE"]
    
    set secure_val [xget_hw_parameter_value $mhsinst "C_M_AXI_SECURE"]
    set secure_len [string length $secure_val]
    set secure_none [string equal -nocase -length $secure_len $secure_val $ZERO_B1]
    
    set high_addr_val [xget_hw_parameter_value $mhsinst "C_M_AXI_HIGH_ADDR"]
    set high_addr_upper [string range $high_addr_val 0 end-16]
    set upper_len [string length $high_addr_upper]
    set ZERO_RANGES "0x[string repeat "0" $upper_len]"
    set upper_ranges_null [string equal -nocase -length $upper_len $high_addr_upper $ZERO_RANGES]
    
    set num_s [xget_hw_parameter_value $mhsinst "C_NUM_SLAVE_SLOTS"]
    set s_protocol_val [xget_hw_parameter_value $mhsinst "C_S_AXI_PROTOCOL"]
    set any_s_full 0
    for {set i 0} {$i<$num_s} {incr i} {
      set low_val [string range $s_protocol_val end end]
      if {[string equal $low_val $AXILITE] == 0} {
        set any_s_full 1
      }
      set s_protocol_val [string range $s_protocol_val 0 end-8]
    }

    set num_m [xget_hw_parameter_value $mhsinst "C_NUM_MASTER_SLOTS"]
    set m_protocol_val [xget_hw_parameter_value $mhsinst "C_M_AXI_PROTOCOL"]
    set any_m_lite 0
    for {set i 0} {$i<$num_m} {incr i} {
      set low_val [string range $m_protocol_val end end]
      if {[string equal $low_val $AXILITE] != 0} {
        set any_m_lite 1
      }
      set m_protocol_val [string range $m_protocol_val 0 end-8]
    }
    
    if {$secure_none == 0 || $upper_ranges_null == 0 || ($any_s_full != 0 && $any_m_lite != 0)} {
      puts "\nINFO: Setting C_RANGE_CHECK = ON for axi_interconnect ${instname}."
      return $ON
    } else {
      puts "\nINFO: Setting C_RANGE_CHECK = OFF for axi_interconnect ${instname}; no DECERR checking will be performed."
      return $OFF
    }
}

#
# Entry point for PLATGEN_SYSLEVEL_UPDATE_PROC
#
proc platgen_update  {mhsinst} {
    generate_corelevel_ucf $mhsinst
    generate_corelevel_xdc $mhsinst
}

##
# generate_corelevel_ucf
# 
proc generate_corelevel_ucf {mhsinst} {

  # Open pcore UCF file for writing
    set  filePath [xget_ncf_dir $mhsinst]
    file mkdir    $filePath
    set    instname   [xget_hw_parameter_value $mhsinst "INSTANCE"]
    set    name_lower [string   tolower   $instname]
    set    fileName   $name_lower
    append fileName   "_wrapper.ucf"
    append filePath   $fileName
    set    outputFile [open $filePath "w"]
    
  # Clock frequency related parameters
    set P_MAX_NUM_SLOTS 16
    set num_s [xget_hw_parameter_value $mhsinst "C_NUM_SLAVE_SLOTS"]
    set num_m [xget_hw_parameter_value $mhsinst "C_NUM_MASTER_SLOTS"]
    set P_ACLK_ASYNC_ZERO "0b[string repeat "0" ${P_MAX_NUM_SLOTS}]"
    set s_aclk_async [xget_hw_parameter_value $mhsinst "C_S_AXI_IS_ACLK_ASYNC"]    
    set m_aclk_async [xget_hw_parameter_value $mhsinst "C_M_AXI_IS_ACLK_ASYNC"]    
    set s_async_zero [string equal -nocase $s_aclk_async $P_ACLK_ASYNC_ZERO]
    set m_async_zero [string equal -nocase $m_aclk_async $P_ACLK_ASYNC_ZERO]
    set s_aclk_ratio [xget_hw_parameter_value $mhsinst "C_S_AXI_ACLK_RATIO"]    
    set m_aclk_ratio [xget_hw_parameter_value $mhsinst "C_M_AXI_ACLK_RATIO"]    
    set ict_aclk_ratio [xget_hw_parameter_value $mhsinst "C_INTERCONNECT_ACLK_RATIO"]    
    set ict_aclk_port [xget_hw_port_handle $mhsinst "INTERCONNECT_ACLK"]
    set ict_aclk_freq [xget_hw_subproperty_value $ict_aclk_port "CLK_FREQ_HZ"]
    set any_sync_conv 0
    set s_temp_async $s_aclk_async
    set m_temp_async $m_aclk_async
    set s_temp_ratio $s_aclk_ratio
    set m_temp_ratio $m_aclk_ratio
    
  # INTERCONNECT_ARESETN input is resynchronized and is excluded from timing analysis.
    puts $outputFile "INST \"${instname}/*clock_conv*/*_resync*\" TNM = FFS \"${instname}_reset_resync\";"
    puts $outputFile "TIMEGRP \"${instname}_reset_source\" = FFS PADS CPUS;"
    puts $outputFile "TIMESPEC \"TS_${instname}_reset_resync\" = FROM \"${instname}_reset_source\" TO \"${instname}_reset_resync\" TIG;"
    
  # Sync clock conversions no longer require timing constraints, as of v1.06.a.

  # Write tspecs for async clock conversion only if async fifos instantiated
    if { $s_async_zero == 1 && $m_async_zero == 1} {
      puts "INFO: No asynchronous clock conversions in axi_interconnect ${instname}."
    } else {
      puts "INFO: Generating core-level timing constraints for asynchronous clock conversions in axi_interconnect ${instname}."
      puts $outputFile "INST \"${instname}/*_converter_bank/*clock_conv_inst/*asyncfifo_*/*mem/*dout_i_?\" TNM = \"${instname}_async_clock_conv_FFDEST\";"
      puts $outputFile "INST \"${instname}/*_converter_bank/*clock_conv_inst/*asyncfifo_*/*mem/*dout_i_??\" TNM = \"${instname}_async_clock_conv_FFDEST\";"
      puts $outputFile "INST \"${instname}/*_converter_bank/*clock_conv_inst/*asyncfifo_*/*mem/*dout_i_???\" TNM = \"${instname}_async_clock_conv_FFDEST\";"
      puts $outputFile "INST \"${instname}/*_converter_bank/*clock_conv_inst/*asyncfifo_*/*mem/*dout_i_????\" TNM = \"${instname}_async_clock_conv_FFDEST\";"
      if { [llength $ict_aclk_freq ] == 0 } { 
        warning "Cannot determine frequency of INTERCONNECT_ACLK for axi_interconnect ${instname}; generating TIG constraint for asynchronous clock-converter data pathways."
        puts $outputFile "TIMESPEC \"TS_${instname}_async_clock_conv\" = FROM RAMS TO \"${instname}_async_clock_conv_FFDEST\" TIG;"
      } else {
        set ict_aclk_period_ps [expr 1000000000000 / $ict_aclk_freq]
        puts $outputFile "TIMESPEC \"TS_${instname}_async_clock_conv\" = FROM RAMS TO \"${instname}_async_clock_conv_FFDEST\" ${ict_aclk_period_ps} PS DATAPATHONLY;"
      }
      puts $outputFile "NET \"${instname}/*_converter_bank/*clock_conv_inst/*asyncfifo_*/wr_pntr_gc*\" TIG;"
      puts $outputFile "NET \"${instname}/*_converter_bank/*clock_conv_inst/*asyncfifo_*/rd_pntr_gc*\" TIG;"
      puts $outputFile "NET \"${instname}/*_converter_bank/*clock_conv_inst/*asyncfifo_*/*clkx/wr_q*\" TIG;"
      puts $outputFile "NET \"${instname}/*_converter_bank/*clock_conv_inst/*asyncfifo_*/*clkx/rd_q*\" TIG;"
      puts $outputFile "NET \"${instname}/*_converter_bank/*clock_conv_inst/*async_conv_reset\" TIG;"
    }

    close $outputFile
}
##
# generate_corelevel_xcd
# 
proc generate_corelevel_xdc {mhsinst} {

  # Open pcore XDC file for writing
    set P_MAX_NUM_SLOTS 16
    set  filePath [xget_ncf_dir $mhsinst]
    file mkdir    $filePath
    set    instname   [xget_hw_parameter_value $mhsinst "INSTANCE"]
    set    name_lower [string   tolower   $instname]
    set    fileName   $name_lower
    append fileName   ".xdc"
    append filePath   $fileName
    file delete -force ${filePath}
    set    xdc [open $filePath "w"]
    
    puts $xdc "###############################################################################################################"
    puts $xdc "# Core-Level Timing Constraints for AXI Interconnect Component ${instname}"
    puts $xdc "###############################################################################################################"
    puts $xdc "#"
    puts $xdc "# Global timing constraints:"
    puts $xdc "set_false_path -through \[get_ports \"INTERCONNECT_ARESETN\"]"
    set num_slots_list [list [xget_hw_parameter_value $mhsinst "C_NUM_SLAVE_SLOTS"] [xget_hw_parameter_value $mhsinst "C_NUM_MASTER_SLOTS"]]
    set ifc_list [list "S" "M"]
    set P_ACLK_ASYNC_ZERO "0b[string repeat "0" ${P_MAX_NUM_SLOTS}]"
    set any_async 0 
    foreach ifc_i {0 1} {
      set ifc_prefix [lindex $ifc_list $ifc_i]
      set aclk_async_string [xget_hw_parameter_value $mhsinst "C_${ifc_prefix}_AXI_IS_ACLK_ASYNC"]    
      if {![string equal -nocase $aclk_async_string $P_ACLK_ASYNC_ZERO]} {
        set any_async 1
      }
    }
    if {$any_async} {
      puts $xdc "#                                                                                                    "
      puts $xdc "# This component is configured to perform asynchronous clock-domain-crossing.                        "
      puts $xdc "# In order for these core-level constraints to work properly,                                        "
      puts $xdc "# the following rules apply to your system-level timing constraints:                                 "
      puts $xdc "#   1. Each of the nets connected to the INTERCONNECT_ACLK, S_AXI_ACLK and M_AXI_ACLK ports of this  "
      puts $xdc "#      component must have exactly one clock defined on it, using either                             "
      puts $xdc "#      a) a create_clock command on a top-level clock pin specified in your system XDC file, or      "
      puts $xdc "#      b) a create_generated_clock command, typically generated automatically by a core              "
      puts $xdc "#          producing a derived clock signal.                                                         "
      puts $xdc "#   2. Any S_AXI_ACLK or M_AXI_ACLK ports of this component that are associated with an asynchronous "
      puts $xdc "#      clock conversion should not be connected to the same clock source as INTERCONNECT_ACLK.       "
      puts $xdc "#                                                                                                    "
      puts $xdc "set INTERCONNECT_ACLK \[get_clocks -of_objects \[get_ports INTERCONNECT_ACLK]]"
      puts $xdc "set_max_delay -from \[get_cells -hierarchical -filter \{NAME =~ *_async_conv_reset_reg\}] -datapath_only \[get_property PERIOD \$INTERCONNECT_ACLK]"
      puts $xdc "set_disable_timing -from CLK -to O \[filter \[get_cells -hierarchical -filter \{NAME =~ *asyncfifo*/*RAM_reg*/RAM*\}] \{REF_NAME =~ RAM*\}]"
      puts $xdc "set_max_delay -from \[get_cells -hierarchical -filter \{NAME =~ *asyncfifo*/rd_pntr_gc_reg\[*]\}] -to \[get_cells -hierarchical -filter \{NAME =~ *asyncfifo*.wr_stg_inst/Q_reg*\}] -datapath_only \[get_property PERIOD \$INTERCONNECT_ACLK]"
      puts $xdc "set_max_delay -from \[get_cells -hierarchical -filter \{NAME =~ *asyncfifo*/wr_pntr_gc_reg\[*]\}] -to \[get_cells -hierarchical -filter \{NAME =~ *asyncfifo*.rd_stg_inst/Q_reg*\}] -datapath_only \[get_property PERIOD \$INTERCONNECT_ACLK]"
    } else {
      puts $xdc "#"
      puts $xdc "# This component is not configured to perform asynchronous clock-domain-crossing."
      puts $xdc "# No further timing core-level constraints are needed."
      puts $xdc "# (Synchronous clock-domain-crossings, if any, remain covered by your system-level PERIOD constraints.)"
    }

    close $xdc
}

#
# Entry point for SYSLEVEL_DRC_PROC
#
proc sys_drc  {mhsinst} {
    sys_drc_range_check $mhsinst
    sys_drc_downsize_ratio $mhsinst
    opt_warnings $mhsinst
}

#
# DRC for C_RANGE_CHECK
#
proc sys_drc_range_check {mhsinst} {
    set ON 1
    set OFF 0
    set MAX_NUM_SLOTS 16
    set AXILITE "2"
    set ZERO_B1 "0b[string repeat "0" $MAX_NUM_SLOTS]"
    set instname [xget_hw_parameter_value $mhsinst "INSTANCE"]
    set range_chk_val [xget_hw_parameter_value $mhsinst "C_RANGE_CHECK"]
    
    if {[llength $range_chk_val] != 0} {
      if {$range_chk_val == $OFF} {
        set secure_val [xget_hw_parameter_value $mhsinst "C_M_AXI_SECURE"]
        set secure_len [string length $secure_val]
        set secure_none [string equal -nocase -length $secure_len $secure_val $ZERO_B1]
    
        set high_addr_val [xget_hw_parameter_value $mhsinst "C_M_AXI_HIGH_ADDR"]
        set high_addr_upper [string range $high_addr_val 0 end-16]
        set upper_len [string length $high_addr_upper]
        set ZERO_RANGES "0x[string repeat "0" $upper_len]"
        set upper_ranges_null [string equal -nocase -length $upper_len $high_addr_upper $ZERO_RANGES]
    
        set num_s [xget_hw_parameter_value $mhsinst "C_NUM_SLAVE_SLOTS"]
        set s_protocol_val [xget_hw_parameter_value $mhsinst "C_S_AXI_PROTOCOL"]
        set any_s_full 0
        for {set i 0} {$i<$num_s} {incr i} {
          set low_val [string range $s_protocol_val end end]
          if {[string equal $low_val $AXILITE] == 0} {
            set any_s_full 1
          }
          set s_protocol_val [string range $s_protocol_val 0 end-8]
        }

        set num_m [xget_hw_parameter_value $mhsinst "C_NUM_MASTER_SLOTS"]
        set m_protocol_val [xget_hw_parameter_value $mhsinst "C_M_AXI_PROTOCOL"]
        set any_m_lite 0
        for {set i 0} {$i<$num_m} {incr i} {
          set low_val [string range $m_protocol_val end end]
          if {[string equal $low_val $AXILITE] != 0} {
            set any_m_lite 1
          }
          set m_protocol_val [string range $m_protocol_val 0 end-8]
        }
    
        if {$secure_none == 0} {
          error "\nParameter C_RANGE_CHECK may not be forced OFF (C_RANGE_CHECK = $OFF) if any slaves are configured as secure (C_M_AXI_SECURE != 0).\n" "" "mdt_error"
          return
        }
        if {$num_m > 1} {
          warning "C_RANGE_CHECK is forced OFF and there are more than one slave connected to axi_interconnect ${instname}. Any transaction with an illegal ADDR may produce unpredictable results."
        }
        if {$any_s_full != 0 && $any_m_lite != 0} {
          warning "C_RANGE_CHECK is forced OFF and there are AXI4LITE slaves and non-AXI4LITE masters connected to axi_interconnect ${instname}. Any transaction with burst-length > 1 beat or wider than 32-bits targeting an AXI4LITE slave may produce unpredictable results."
        }
      }
    }
}

#
# DRC for downsize ratio >= 1024:32 (post-PLATGEN)
#
proc sys_drc_downsize_ratio {mhsinst} {
    set instname [xget_hw_parameter_value $mhsinst "INSTANCE"]
    set num_s [xget_hw_parameter_value $mhsinst "C_NUM_SLAVE_SLOTS"]
    set num_m [xget_hw_parameter_value $mhsinst "C_NUM_MASTER_SLOTS"]
    set s_width_param [xget_hw_parameter_value $mhsinst "C_S_AXI_DATA_WIDTH"]
    set m_width_param [xget_hw_parameter_value $mhsinst "C_M_AXI_DATA_WIDTH"]
    set ict_width_param [xget_hw_parameter_value $mhsinst "C_INTERCONNECT_DATA_WIDTH"]
    
    if {[llength $ict_width_param] != 0} {
      if {$ict_width_param == 32 && [llength $s_width_param] != 0 && [llength $num_s] != 0} {
        for {set i 0} {$i<$num_s} {incr i} {
          set low_val [string range $s_width_param end-7 end]
          set hex_val "0x${low_val}"
          if {$hex_val >= 0x400} {
            error "\nInterconnect does not support downsizing directly from 1024-bit (SI slot $i) to 32-bit (interconnect native width).\n" "" "mdt_error"
            return
          }
          set s_width_param [string range $s_width_param 0 end-8]
        }
      }
      if {$ict_width_param >= 1024 && [llength $m_width_param] != 0 && [llength $num_m] != 0} {
        for {set i 0} {$i<$num_m} {incr i} {
          set low_val [string range $m_width_param end-7 end]
          set hex_val "0x${low_val}"
          if {$hex_val == 0x20} {
            error "\nInterconnect does not support downsizing directly from 1024-bit (interconnect native width) to 32-bit (MI slot $i).\n" "" "mdt_error"
            return
          }
          set m_width_param [string range $m_width_param 0 end-8]
        }
      }
    }
}

#
# Optimization warnings to help configuration ease-of-use
#
proc opt_warnings {mhsinst} {
    set instname [xget_hw_parameter_value $mhsinst "INSTANCE"]
    set num_s [xget_hw_parameter_value $mhsinst "C_NUM_SLAVE_SLOTS"]
    set num_m [xget_hw_parameter_value $mhsinst "C_NUM_MASTER_SLOTS"]
    set s_width_param [xget_hw_parameter_value $mhsinst "C_S_AXI_DATA_WIDTH"]
    set m_width_param [xget_hw_parameter_value $mhsinst "C_M_AXI_DATA_WIDTH"]
    set ict_width_param [xget_hw_parameter_value $mhsinst "C_INTERCONNECT_DATA_WIDTH"]
    set s_aclk_async [xget_hw_parameter_value $mhsinst "C_S_AXI_IS_ACLK_ASYNC"]    
    set m_aclk_async [xget_hw_parameter_value $mhsinst "C_M_AXI_IS_ACLK_ASYNC"]    
    set s_aclk_ratio [xget_hw_parameter_value $mhsinst "C_S_AXI_ACLK_RATIO"]    
    set m_aclk_ratio [xget_hw_parameter_value $mhsinst "C_M_AXI_ACLK_RATIO"]    
    set ict_aclk_ratio [xget_hw_parameter_value $mhsinst "C_INTERCONNECT_ACLK_RATIO"]    
    set s_wfifo_param [xget_hw_parameter_value $mhsinst "C_S_AXI_WRITE_FIFO_DEPTH"]
    set s_rfifo_param [xget_hw_parameter_value $mhsinst "C_S_AXI_READ_FIFO_DEPTH"]
    set m_wfifo_param [xget_hw_parameter_value $mhsinst "C_M_AXI_WRITE_FIFO_DEPTH"]
    set m_rfifo_param [xget_hw_parameter_value $mhsinst "C_M_AXI_READ_FIFO_DEPTH"]
    set s_protocol_param [xget_hw_parameter_value $mhsinst "C_S_AXI_PROTOCOL"]
    set m_protocol_param [xget_hw_parameter_value $mhsinst "C_M_AXI_PROTOCOL"]
    set ict_mode_param [xget_hw_parameter_value $mhsinst "C_INTERCONNECT_CONNECTIVITY_MODE"]
    set s_awreg_param [xget_hw_parameter_value $mhsinst "C_S_AXI_AW_REGISTER"]
    set s_arreg_param [xget_hw_parameter_value $mhsinst "C_S_AXI_AR_REGISTER"]
    set s_wreg_param [xget_hw_parameter_value $mhsinst "C_S_AXI_W_REGISTER"]
    set s_rreg_param [xget_hw_parameter_value $mhsinst "C_S_AXI_R_REGISTER"]
    set s_breg_param [xget_hw_parameter_value $mhsinst "C_S_AXI_B_REGISTER"]
    set m_awreg_param [xget_hw_parameter_value $mhsinst "C_M_AXI_AW_REGISTER"]
    set m_arreg_param [xget_hw_parameter_value $mhsinst "C_M_AXI_AR_REGISTER"]
    set m_wreg_param [xget_hw_parameter_value $mhsinst "C_M_AXI_W_REGISTER"]
    set m_rreg_param [xget_hw_parameter_value $mhsinst "C_M_AXI_R_REGISTER"]
    set m_breg_param [xget_hw_parameter_value $mhsinst "C_M_AXI_B_REGISTER"]
    
      set ict_width_match 0
      set max_mi_width 0
      for {set i 0} {$i<$num_s} {incr i} {
        set low_val [string range $s_width_param end-7 end]
        set hex_val "0x${low_val}"
        if {$hex_val == $ict_width_param} {
          set ict_width_match 1
        }
        set s_width_param [string range $s_width_param 0 end-8]
      }
      for {set i 0} {$i<$num_m} {incr i} {
        set low_val [string range $m_width_param end-7 end]
        set hex_val "0x${low_val}"
        if {$hex_val == $ict_width_param} {
          set ict_width_match 1
        }
        if {$hex_val > $max_mi_width} {
          set max_mi_width $hex_val
        }
        set m_width_param [string range $m_width_param 0 end-8]
      }
      if {$ict_width_match == 0} {
        warning "Optimization Alert: axi_interconnect ${instname}: Interconnect data-width ${ict_width_param} does not match any SI or MI slot. Your configuration may have unnecessary width conversion(s). Typically, best throughput is achieved by setting Interconnect data-width (C_INTERCONNECT_DATA_WIDTH) to match the widest performance-critical MI slot (connected slave)."
      } elseif {$ict_width_param < $max_mi_width} {
        warning "Optimization Alert: axi_interconnect ${instname}: Interconnect data-width ${ict_width_param} is narrower than the widest MI slot (connected slave). Throughput may often be improved by setting Interconnect data-width (C_INTERCONNECT_DATA_WIDTH) to match the widest performance-critical MI slot."
      }
      
      set ict_aclk_match 0
      set max_mi_aclk 0
      set extra_fifo 0
      for {set i 0} {$i<$num_s} {incr i} {
        if {[string range $s_aclk_async end end] == 0} {
          set low_val [string range $s_aclk_ratio end-7 end]
          set hex_val "0x${low_val}"
          if {abs($hex_val - $ict_aclk_ratio) <= 1} {
            set ict_aclk_match 1
          }
        } elseif {[llength $s_wfifo_param ] !=0 && [llength $s_rfifo_param] !=0} {
          if {[string range $s_wfifo_param end-7 end] == "00000020" || [string range $s_rfifo_param end-7 end] == "00000020"} {
            set extra_fifo 1
          }
        }
        set s_aclk_async [string range $s_aclk_async 0 end-1]
        set s_aclk_ratio [string range $s_aclk_ratio 0 end-8]
        set s_wfifo_param [string range $s_wfifo_param 0 end-8]
        set s_rfifo_param [string range $s_rfifo_param 0 end-8]
      }
      for {set i 0} {$i<$num_m} {incr i} {
        if {[string range $m_aclk_async end end] == 0} {
          set low_val [string range $m_aclk_ratio end-7 end]
          set hex_val "0x${low_val}"
          if {abs($hex_val - $ict_aclk_ratio) <= 1} {
            set ict_aclk_match 1
          }
          if {$hex_val > $max_mi_aclk} {
            set max_mi_aclk $hex_val
          }
        } elseif {[llength $m_wfifo_param ] !=0 && [llength $m_rfifo_param] !=0} {
          if {[string range $m_wfifo_param end-7 end] == "00000020" || [string range $m_rfifo_param end-7 end] == "00000020"} {
            set extra_fifo 1
          }
        }
        set m_aclk_async [string range $m_aclk_async 0 end-1]
        set m_aclk_ratio [string range $m_aclk_ratio 0 end-8]
        set m_wfifo_param [string range $m_wfifo_param 0 end-8]
        set m_rfifo_param [string range $m_rfifo_param 0 end-8]
      }
      if {$ict_aclk_match == 0} {
        warning "Optimization Alert: axi_interconnect ${instname}: Port INTERCONNECT_ACLK is not connected to the same clock source as any SI or MI slot. Your configuration may have unnecessary clock conversion(s). Typically, best throughput is achieved by connecting INTERCONNECT_ACLK to the same clock source as the fastest performance-critical MI slot (connected slave)."
      } elseif {abs($max_mi_aclk - $ict_aclk_ratio) > 1} {
        warning "Optimization Alert: axi_interconnect ${instname}: Port INTERCONNECT_ACLK is connected to a clock source slower than (or asynchronous to) the fastest MI slot (connected slave). Throughput may often be improved by connecting INTERCONNECT_ACLK to the same clock source as the fastest performance-critical MI slot."
      }
      if {$extra_fifo} {
        warning "Optimization Alert: axi_interconnect ${instname}: One or more SI or MI pathway has both a 32-deep write or read data-path FIFO and an asynchronous clock converter, which is built using a 32-deep FIFO. The data-path FIFO may be redundant. To save resources, disable C_S/M_AXI_WRITE_FIFO_DEPTH and C_S/M_AXI_READ_FIFO_DEPTH (set to 0) on pathways with async clock conversion."
      }

      set full_axi 0
      set full_reg_lite 0
      set full_reg_ctrl 0
      for {set i 0} {$i<$num_s} {incr i} {
        if {[string range $s_protocol_param end-7 end] == "00000002"} {
          if {[string range $s_wreg_param end-7 end] == "00000001" || [string range $s_rreg_param end-7 end] == "00000001"} {
            set full_reg_lite 1
          }
        } else {
          set full_axi 1
        }
        if {[string range $s_awreg_param end-7 end] == "00000001" || [string range $s_arreg_param end-7 end] == "00000001" || [string range $s_breg_param end-7 end] == "00000001"} {
          set full_reg_ctrl 1
        }
        set s_protocol_param [string range $s_protocol_param 0 end-8]
        set s_awreg_param [string range $s_awreg_param 0 end-8]
        set s_arreg_param [string range $s_arreg_param 0 end-8]
        set s_wreg_param [string range $s_wreg_param 0 end-8]
        set s_rreg_param [string range $s_rreg_param 0 end-8]
        set s_breg_param [string range $s_breg_param 0 end-8]
      }
      for {set i 0} {$i<$num_m} {incr i} {
        if {[string range $m_protocol_param end-7 end] == "00000002"} {
          if {[string range $m_wreg_param end-7 end] == "00000001" || [string range $m_rreg_param end-7 end] == "00000001"} {
            set full_reg_lite 1
          }
        } else {
          set full_axi 1
        }
        if {[string range $m_awreg_param end-7 end] == "00000001" || [string range $m_arreg_param end-7 end] == "00000001" || [string range $m_breg_param end-7 end] == "00000001"} {
          set full_reg_ctrl 1
        }
        set m_protocol_param [string range $m_protocol_param 0 end-8]
        set m_awreg_param [string range $m_awreg_param 0 end-8]
        set m_arreg_param [string range $m_arreg_param 0 end-8]
        set m_wreg_param [string range $m_wreg_param 0 end-8]
        set m_rreg_param [string range $m_rreg_param 0 end-8]
        set m_breg_param [string range $m_breg_param 0 end-8]
      }
      if {$full_axi == 0 && $ict_mode_param > 0} {
        warning "Optimization Alert: axi_interconnect ${instname}: All SI and MI slots use AXI4-Lite protocol, and the interconnect is configured as a high-performance crossbar (C_INTERCONNECT_CONNECTIVITY_MODE = 1). AXI4-Lite traffic generally cannot take advantage of full crossbar functionality. To reduce resources, set C_INTERCONNECT_CONNECTIVITY_MODE = 0 (\"Shared-access\")."
      }
      if {$full_reg_lite} {
        warning "Optimization Alert: axi_interconnect ${instname}: One or more SI or MI slots uses AXI4-Lite protocol and has a W or R channel register slice configured as \"Fully-registered\" (C_S/M_AXI_W/R_REGISTER = 1). AXI4-Lite data transfers generally cannot take advantage of fully-registered performance. To reduce resources, use \"Light-weight\" reg-slices on all channels of AXI4-Lite slots (C_S/M_AXI_*_REGISTER = 7)."
      }
      if {$full_reg_ctrl} {
        warning "Optimization Alert: axi_interconnect ${instname}: One or more SI or MI slots uses register slices configured as \"Fully-registered\" on their AW, AR or B channels (C_S/M_AXI_AW/AR/B_REGISTER = 1). These control channels generally cannot take advantage of fully-registered performance. To reduce resources, use \"Light-weight\" reg-slices on AW, AR or B channels (C_S/M_AXI_AW/AR/B_REGISTER = 7)."
      }
}
