###################################################################
## chipscope_icon
##
## Copyright(C) 2003 by Xilinx, Inc. All rights reserved.
##
## @BEGIN_CHANGELOG EDK_H
##
## Updated Tcl lookup mechanism for finding path to ChipScope Core Generator installation
##
## @END_CHANGELOG
##
## @BEGIN_CHANGELOG EDK_I
##
##    - Support for auto detection of OPB_MDM and auto connection to the unused BSCAN_User signals
##       in case of pre-Virtex4 devices and separate instantiation of Bscan_User1 primitve in case of Virtex4
##    - Support for forcing the use of Bscan_User3/4 for Virtex4 if needed
##    - Support for disabling of JTAG Clock BUFG instantiation
##    - First check for ChipScope in User's PATH before looking at Windows Registry
##
## @END_CHANGELOG
##
## @BEGIN_CHANGELOG EDK_I_SP1
##
##    - Fixed problem on non-Virtex4 parts for automatic inference of C_SYSTEM_CONTAINS_MDM parameter
##
## @END_CHANGELOG
## 
## 
## @BEGIN_CHANGELOG EDK_Im
##
##    - Added support for Virtex5
##    - Added lin64 support
##
## @END_CHANGELOG
##
## @BEGIN_CHANGELOG EDK_L
##
##    - Added support for Virtex6 and Spartan6
##
## @END_CHANGELOG
## 
## @BEGIN_CHANGELOG EDK_O
##
##    - Added support for Virtex7, Kintex7 and Artex7
##
## @END_CHANGELOG
##
###################################################################

proc icon_simulate { mhsinst } {
  
}

proc icon_drc { mhsinst } {
  
  set param_names { "INSTANCE" "HW_VER" "C_FAMILY" "C_DEVICE" "C_PACKAGE"
    "C_SPEEDGRADE" "C_NUM_CONTROL_PORTS" "C_SYSTEM_CONTAINS_MDM" 
    "C_FORCE_BSCAN_USER_PORT" }
  set param_table {}
  
  foreach name $param_names {
    lappend param_table $name
    lappend param_table [xget_value $mhsinst "PARAMETER" $name]
  }
  
  # put the PARAMETER name value pairs into an associative array 
  # for easy access Ex: set instance_name $params(INSTANCE)
  array set params $param_table
  
  # Control ports must be used sequentially without gaps
  # beginning with control0, for example if you use control1
  # but not control0, XST will fail.
  for { set i 0 } { $i < $params(C_NUM_CONTROL_PORTS) } { incr i } {
    set control_port "control$i"
    check_control_port_is_connected $mhsinst $control_port
  }
}

proc check_control_port_is_connected { mhsinst pin } {
  
  set connector          [xget_hw_port_value      $mhsinst $pin]
  
  if {[llength $connector] == 0} {
    error  "ERROR: CHIPSCOPE_ICON control port $pin is unconnected - control ports must be assigned sequentially beginning with control0 without gaps in numbering.\n"
  }
}

proc syslevel_update_mdm_param { param_handle } {
  
  set mhsinst    [xget_handle $param_handle "parent"]
  set mhs_handle [xget_handle $mhsinst "parent"]
  
  set iplist [xget_hw_ipinst_handle $mhs_handle "*"]
  
  foreach ip $iplist {
    set ip_instance_name [xget_hw_name $ip]
    set ip_realname [xget_hw_value $ip]
    # puts "Instance $ip_instance_name has realname: $ip_realname"
    if { $ip_realname == "opb_mdm" || $ip_realname == "mdm" } {
      return 1
    }
  }
  # Unable to find MDM in the design
  return 0    
}

proc icon_generate { mhsinst } {
  global errorCode errorInfo
  
  #puts "**********************************************"
  #puts "**********************************************"
  
  set param_names { "INSTANCE" "HW_VER" "C_FAMILY" "C_DEVICE" "C_PACKAGE"
    "C_SPEEDGRADE" "C_NUM_CONTROL_PORTS" "C_SYSTEM_CONTAINS_MDM" 
    "C_FORCE_BSCAN_USER_PORT" }
  set param_table {}
  
  foreach name $param_names {
    lappend param_table $name
    lappend param_table [xget_value $mhsinst "PARAMETER" $name]
  }
  if { ( [xget_value $mhsinst "PARAMETER" "C_SYSTEM_CONTAINS_MDM"] == 1) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "virtex4" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "virtex5" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "spartan6" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "virtex6" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "virtex6l" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "kintex7" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "virtex7" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "zynq" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "artix7" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "virtex7l" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "kintex7l" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "artix7l" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "aartix7" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qkintex7l" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qkintex7" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qvirtex7l" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qvirtex7" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qartix7l" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qartix7" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qzynq" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "azynq" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qspartan6" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qspartan6l" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qvirtex5" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qvirtex6" ) &&
       ( [xget_value $mhsinst "PARAMETER" "C_FAMILY"] != "qvirtex6l" )} {
    lappend param_table "REUSE_BSCAN_USER1_PORT_FROM_MDM"
    lappend param_table 1
  } else {
    lappend param_table "REUSE_BSCAN_USER1_PORT_FROM_MDM"
    lappend param_table 0 
  }
  
  # puts "Param values are : $param_table"
  
  # put the PARAMETER name value pairs into an associative array 
  # for easy access Ex: set instance_name $params(INSTANCE)
  array set params $param_table
  
  
  # First Create the HDL wrapper to instantiate the ICON core
  # This intermediate wrapper is needed to match the port declaration
  # (no of ports is fixed) in the EDK ICON core and the port declaration
  # in the ChipScope core generated by the ChipScope Core Generator engine
  # Wrapper location:
  # <proj>/hdl/elaborate/icon_v1_01_a/hdl/vhdl/icon_wrap.vhd
  
  set params(INSTANCE) [string tolower $params(INSTANCE)]
  write_hdl_wrapper $mhsinst $param_table
  
  # Second, write out a ChipScope Core Generator xco file that matches
  # the parameters set by the user.
  
  set netlist_directory [file nativename [file join [pwd] implementation "$params(INSTANCE)_wrapper"]]
    
  set xco_file [write_xco_file $param_table $netlist_directory]
  
  # check if parameters are the same. If so, dont regenerate the CS core
  # Compare .xco file and previous .xco.last file to check for equality
  
  if { [file exists $xco_file.last] } {
    if { [file size $xco_file] == [file size $xco_file.last] } {
      set xcof_cur [open $xco_file]
      set xcof_last [open $xco_file.last]
      while { ![eof $xcof_cur] } {
        if { [gets $xcof_cur] != [gets $xcof_last] } {
          break
        }
      }
      if { [eof $xcof_cur] } {
        puts "INFO: ChipScope core parameters not changed. Not regenerating core"
        close $xcof_cur
        close $xcof_last
        return
      }
      close $xcof_cur
      close $xcof_last
      file delete -force $xco_file.last
    }
  }
  
  # Third, call the ChipScope Core Generator with the xco file to create
  # the core netlist in <proj>/implementation/CORE_INSTANCENAME/.
  # Platgen will run ngcbuild from this location to merge the netlists 
  # into a single NGC file. This helps in case of multiple instantiations
  # of the same core.
  set instance_name [string tolower $params(INSTANCE)]
  set params(INSTANCE) [string tolower $params(INSTANCE)]
#  set instance_name $params(INSTANCE)
  puts "Generating ChipScope core: $params(INSTANCE) ..."
  
  
  # Start with clean directory for Coregen
  if {[file exists $netlist_directory]} {
    file delete -force $netlist_directory
  }
  file mkdir $netlist_directory

  if { [catch {generate_chipscope_core $xco_file $instance_name $netlist_directory} errMsg] } {
    
    error $errMsg
    
  }
 
  puts "*************************************************************"
  #puts "Successfully generated the ChipScope Core : $params(INSTANCE)"
  #puts "*************************************************************"
  
  # Keep a copy of the xco file for comparison next time. This is the caching mechanism.
  file copy -force $xco_file $xco_file.last
  
  ## TODO : See what the NCF error is 
  #    exec rm -f [file nativename [pwd]]/implementation/$params(INSTANCE)_wrapper/$params(INSTANCE).ncf
  return
}

proc write_hdl_wrapper { handle param_table } {
  
  # put the PARAMETER name value pairs into an associative array 
  # for easy access Ex: set instance_name $params(INSTANCE)
  array set params $param_table
  
  set params(INSTANCE) [string tolower $params(INSTANCE)]

  set hdllib $params(INSTANCE)
  set hw_ver [string map {. _} [xget_hw_parameter_value $handle "HW_VER"]]
  if { [string length $hw_ver] != 0 } {
    append hdllib "_v" $hw_ver
  }
  
  set hdldir "hdl/elaborate/$hdllib/synhdl/vhdl/"
  if {![file exists $hdldir]} {
    file mkdir $hdldir
  }
  puts "Created elaborate directory $hdldir"
  
  set filename "$params(INSTANCE).vhd"
  set filepath [file join $hdldir/$filename]
  
  set hdl_file [open $filepath "w"]
  
  # Update PAO
  # xadd_hw_hdl_srcfile <ipinst_handle> <fileuse> <filename> <hdllang>
  xadd_hw_hdl_srcfile $handle "synlib" $filename "VHDL"
  
  puts $hdl_file "-- ICON Wrapper HDL file generated by icon's TCL generator"
  puts $hdl_file "
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity chipscope_icon is
  generic (
    C_FAMILY : string := \"virtex5\" ;
    C_NUM_CONTROL_PORTS : integer := $params(C_NUM_CONTROL_PORTS);
    C_SYSTEM_CONTAINS_MDM : integer := $params(C_SYSTEM_CONTAINS_MDM);
    C_FORCE_BSCAN_USER_PORT : integer := $params(C_FORCE_BSCAN_USER_PORT)
  );
  port (
    control0   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control1   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control2   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control3   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control4   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control5   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control6   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control7   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control8   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control9   : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control10  : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control11  : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control12  : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control13  : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control14  : out STD_LOGIC_VECTOR ( 35 downto 0 ); 
    control15  : out STD_LOGIC_VECTOR ( 35 downto 0 );
    tdi_in     : in  STD_LOGIC;
    reset_in   : in  STD_LOGIC;
    shift_in   : in  STD_LOGIC;
    update_in  : in  STD_LOGIC;
    sel_in     : in  STD_LOGIC;
    drck_in    : in  STD_LOGIC;
    capture_in : in std_logic;
    tdo_out    : out STD_LOGIC
  );
end chipscope_icon;

architecture STRUCTURE of chipscope_icon is

  attribute box_type : STRING;

  component $params(INSTANCE) is
    port ("
  
  if { $params(REUSE_BSCAN_USER1_PORT_FROM_MDM) == 1 } {
    puts $hdl_file \
        "      tdi_in    : in STD_LOGIC;
      reset_in   : in STD_LOGIC;
      shift_in   : in STD_LOGIC;
      update_in  : in STD_LOGIC;
      sel_in     : in STD_LOGIC;
      drck_in    : in STD_LOGIC;
      capture_in : in std_logic;
      tdo_out    : out STD_LOGIC;"
  }
  
  for { set i 0 } { $i < $params(C_NUM_CONTROL_PORTS) } { incr i } {
    puts -nonewline $hdl_file "      control$i  : out std_logic_vector ( 35 downto 0)"
    if { $i < $params(C_NUM_CONTROL_PORTS) - 1 } {
      puts $hdl_file ";"
    }
  }
  
  puts $hdl_file "
    );
 end component;
 attribute box_type of $params(INSTANCE) : component is \"black_box\";

begin

  i_$params(INSTANCE) : $params(INSTANCE)
    port map ("
  
  if { $params(REUSE_BSCAN_USER1_PORT_FROM_MDM) == 1 } {
    puts $hdl_file \
        "      tdi_in => tdi_in,
      reset_in => reset_in,
      shift_in => shift_in,
      update_in => update_in,
      sel_in => sel_in,
      drck_in => drck_in,
      capture_in => capture_in,
      tdo_out => tdo_out,"
  }
  
  for { set i 0 } { $i < $params(C_NUM_CONTROL_PORTS) } { incr i } {
    puts -nonewline $hdl_file "      control$i => control$i"
    if { $i < $params(C_NUM_CONTROL_PORTS) - 1 } {
      puts $hdl_file ","
    }
  }
  puts $hdl_file "
    );
"
  
  for { set i $params(C_NUM_CONTROL_PORTS) } { $i < 16 } { incr i} {
    puts $hdl_file "  control$i <= (others => '0');"
  }
  
  puts $hdl_file "

end architecture STRUCTURE;
"
  close $hdl_file
}

proc write_xco_file { param_table netlist_directory } {
  global tcl_platform 
  
  # put the PARAMETER name value pairs into an associative array 
  # for easy access Ex: set instance_name $params(INSTANCE)
  array set params $param_table

  set params(INSTANCE) [string tolower $params(INSTANCE)]
  
#  set filename "implementation/$params(INSTANCE).xco"
  set filename [file join [pwd] "implementation" [format "%s.xco" $params(INSTANCE)]]
  set xcofile [open $filename "w"]
  
  puts $xcofile "####################################################################"
  puts $xcofile "##  ChipScope ICON XCO file generated by chipscope_icon_v2_1_0.tcl #"
  puts $xcofile "####################################################################"
  
  puts $xcofile "NEWPROJECT $netlist_directory"
  puts $xcofile "SETPROJECT $netlist_directory"
  
  puts $xcofile "SET devicefamily = $params(C_FAMILY)"
  puts $xcofile "SET device = $params(C_DEVICE)"
  puts $xcofile "SET package = $params(C_PACKAGE)"
  puts $xcofile "SET speedgrade = $params(C_SPEEDGRADE)"
  
  puts $xcofile "SET simulationfiles = structural"
  puts $xcofile "SET vhdlsim = False"
  puts $xcofile "SET verilogsim = False"
  
  puts $xcofile "SELECT ICON_(ChipScope_Pro_-_Integrated_Controller) family Xilinx,_Inc. 1.06.a"
  
  puts $xcofile "CSET component_name = $params(INSTANCE)"
  puts $xcofile "CSET number_control_ports = $params(C_NUM_CONTROL_PORTS)"
  
  puts -nonewline $xcofile "CSET use_ext_bscan = "
  if { $params(REUSE_BSCAN_USER1_PORT_FROM_MDM) == 1 } {
    puts $xcofile "true"
  } else {
    puts $xcofile "false"
    puts -nonewline $xcofile "CSET user_scan_chain = USER"
    puts $xcofile "$params(C_FORCE_BSCAN_USER_PORT)"
  }
  
  puts $xcofile "CSET enable_jtag_bufg = true"
  
  puts $xcofile "GENERATE"
  
  close $xcofile
  
  return $filename
}

#***--------------------------------***-----------------------------------***
# Utility process to call a command and pipe it's output to screen.
# Used instead of Tcl's exec
proc execpipe {COMMAND} {

  if { [catch {open "| $COMMAND 2>@stdout"} FILEHANDLE] } {
    return "Can't open pipe for '$COMMAND'"
  }

  set PIPE $FILEHANDLE
  fconfigure $PIPE -buffering none

  set OUTPUT ""

  while { [gets $PIPE DATA] >= 0 } {
    puts $DATA
    append OUTPUT $DATA "\n"
  }

  if { [catch {close $PIPE} ERRORMSG] } {

    if { [string compare "$ERRORMSG" "child process exited abnormally"] == 0 } {
      # this error means there was nothing on stderr (which makes sense) and
      # there was a non-zero exit code - this is OK as we intentionally send
      # stderr to stdout, so we just do nothing here (and return the output)
    } else {
      return "Error '$ERRORMSG' on closing pipe for '$COMMAND'"
    }

  }

  regsub -all -- "\n$" $OUTPUT "" STRIPPED_STRING
  return "$STRIPPED_STRING"

}

#***--------------------------------***-----------------------------------***
proc generate_chipscope_core { xco_file instance_name netlist_directory } {
  global tcl_platform env tcl_library
  
  switch -glob $tcl_platform(os) {
    "Windows*" {
      if { [auto_execok coregen] == ""} {
        error "Unable to find the ChipScope Core Generator \"coregen.exe\" in the path. Please ensure that a valid ChipScope Installation exists."
      }
      set coregen_cmd "coregen.exe"
    }
    
    "Linux" {
      if { [auto_execok coregen] == ""} {
        error "Unable to find the ChipScope Core Generator \"coregen\" in the path. Please ensure that a valid ChipScope Installation exists."
      }
      set coregen_cmd "coregen"
    }
    
    default {
      error "
****************************************************************************
Instantiation of ChipScope cores is not supported on $tcl_platform(os)
The ChipScope Core Generator engine \"generate.sh\" is not available on $tcl_platform(os).
To fix the problem remove this core from the MHS file
OR use Windows or Solaris platform to instantiate this core
****************************************************************************" "Platform Not Supported" "CS_UNAVAILABLE"
    }
  }
  
  # remove ngc (about to generate the file through coregen)
  set ngc_filename [file join $netlist_directory [format "%s.ngc" $instance_name]]
  file delete -force $ngc_filename
  
  # prepare to run the command. it will be run from $netlist_directory.
  # this is done so the log file is created in the netlist directory.
  set current_pwd [pwd]
  cd $netlist_directory

  # run the command
  puts -nonewline "ChipScope Core Generator command: "
  puts "$coregen_cmd -b $xco_file"
  execpipe "$coregen_cmd -b $xco_file"
  cd $current_pwd

  # check for output ngc file
  if {![file exists $ngc_filename]} {
    error "ERROR: Coregen did not generate file \"$ngc_filename\"" ""
  }

##############
set srcNcfFile [file join $netlist_directory [format "%s.ncf" $instance_name]]

set wrapper_UcfFile $instance_name
append wrapper_UcfFile "_wrapper.ucf"

set tgtUcfFile [file join $netlist_directory $wrapper_UcfFile]

file rename -- $srcNcfFile $tgtUcfFile
##############

  return
}

