###################################################################
##
## Copyright(C) 2003-2006 by Xilinx, Inc. All rights reserved.
##
## chipscope_ila
##
###################################################################
##
## @BEGIN_CHANGELOG EDK_H
##
## Updated Tcl lookup mechanism for finding path to Chipscope Core Generator installation
##
## @END_CHANGELOG
##
###################################################################
## @BEGIN_CHANGELOG EDK_I
##
##    - Support for Multiple ILA instantiation
##    - Added parameters for 
##          1) Disabling SRL16s
##          2) Support for Rising/Falling clock setup
##          3) Support for Trigger Sequencer (Enabled by default)
##          4) Support for Storage Qualification (Enabled by default)
##    - First check for Chipscope in User's PATH before looking at Windows Registry
##
## @END_CHANGELOG
##
##
## @BEGIN_CHANGELOG EDK_Im
##
##    - Added support for Virtex5
##
## @END_CHANGELOG
##
## @BEGIN_CHANGELOG EDK_J
##
##    - Added support for all possible trigger units
##    - Update CDC file to create busses for bus signals
##    - Range for depth check depends on device family
##    - DRCs for architecture dependent parameters
##      C_DISABLE_SRL16S & C_DISABLE_RPM
##
## @END_CHANGELOG
##
## @BEGIN_CHANGELOG EDK_Jm
##
##    - SRL16 type updated for Spartan3A, Spartan3ADSP
##    - add lin64 support
##
## @END_CHANGELOG
##
## @BEGIN_CHANGELOG EDK_L
##
##    - Added support for Virtex6 and Spartan6
##
## @END_CHANGELOG
##
###################################################################

#***--------------------------------***-----------------------------------***
#
#          SYSLEVEL_DRC_PROC
#
#***--------------------------------***-----------------------------------***

proc check_syslevel_icon_connect { handle } {

  # This proc checks to see if there is a CHIPSCOP_ICON core instantiated in 
  #   the system. If there is, then it will check if port CHIPSCOPE ILA CONTROL 
  #   is connected to "net_gnd" or "net_vcc" or nothing.  It avoids a generic 
  #   error message provided by the tools and uses a detailed error to let the user 
  #   know that they must connect an ICON core to the ILA 

  
  # Get MHS Handle
  set mhsHandle [xget_hw_parent_handle $handle]
  
  # Get list of all IP instances in system
  set ipInstList [xget_hw_ipinst_handle $mhsHandle "*"]
  
  # Get Chipscope ILA Instance Name (for error message)
  set ilaInst [xget_hw_name $handle]
  
  # Get value of the CHIPSCOPE ILA CONTROL port of the CHIPSCOP_ILA
  set iconPortVal [xget_hw_port_value $handle CHIPSCOPE_ILA_CONTROL]

  # Setting values for checks
  set ipDesc "Chipscope Integrated Controller"
  set gndString "net_gnd"
  set vccString "net_vcc"
  set nullString ""
  
  # Initialize ICON Core found flag  (1 = found, 0 = not found)
  set foundIconCore 0
  
  set msg1 [format "The Chipscope Integrated Controller (Chipscope ICON) core is not instantiated in the system. Chipscope ICON core is needed as 
                    interface between the Chipscope Core in your system and the JTAG BSCAN in order for Chipscope Analyzer Software to read and display data samples captured by the ILA. 
		    Please instantiate the Chipscope ICON core and connect to port CHIPSCOPE_ILA_CONTROL of instance '%s'. Do NOT connect CHIPSCOPE_ILA_CONTROL 
		    port to External Ports through the GUI." $ilaInst]  

  set msg2 [format "The Chipscope Integrated Controller (Chipscope ICON) core is not connected correctly to the instance '%s'. Chipscope ICON core 
                    is needed as interface between the Chipscope Core in your system and the JTAG BSCAN in order for Chipscope Analyzer Software to read and display data samples captured by 
		    the ILA. Currently, port 'CHIPSCOPE_ILA_CONTROL' is connected to '%s'. Do NOT connect this port to External Ports through the GUI. Please connect Chipscope ICON core 
		    to '%s'." $ilaInst $iconPortVal $ilaInst]  
  
  # Loop through all IP instances
  foreach ipInst $ipInstList {
     # Look for CHIPSCOPE_ICON core based on DESCription Option in MPD
     set curIpType [xget_hw_option_value $ipInst "DESC"]
	
      # Found ICON core
      if { [string compare $curIpType $ipDesc] == 0 } {
     	 set iconCore [xget_hw_value $ipInst]
     	 puts [format "Chipscope Integrate Controller, instance '%s', exists in System " $iconCore]
     	 set foundIconCore 1  
      } 
  }
  
  # If not found, print error message 
  if { $foundIconCore == 0 } {
     error $msg1
  } elseif { $foundIconCore == 1 } {
     # Else if the ICON core was found, do additional checks to see if CHIPSCOPE_ILA_CONTROL
     #  port is connected on CHIPSCOPE_ILA
      if { [string match $gndString* $iconPortVal] == 1 } {
        error $msg2
      } elseif { [string match $vccString* $iconPortVal] == 1 } {
        error $msg2
      } elseif { [string match $nullString $iconPortVal] == 1 }  {
        error $msg2
      }
  }

}


#***--------------------------------***-----------------------------------***
#
#			     IPLEVEL_DRC_PROC
#
#***--------------------------------***-----------------------------------***


proc ila_drc { handle } {

	set ipname "chipscope_ila"
	
	set param_names { "INSTANCE" "HW_VER" "C_FAMILY" 
		"C_DEVICE" "C_PACKAGE" "C_SPEEDGRADE"
		"C_NUM_DATA_SAMPLES" "C_ENABLE_TRIGGER_OUT"
		"C_DATA_SAME_AS_TRIGGER" "C_DATA_IN_WIDTH"
		"C_DISABLE_RPM" "C_DISABLE_SRL16S" "C_RISING_CLOCK_EDGE" 
		"C_MAX_SEQUENCER_LEVELS" "C_ENABLE_STORAGE_QUALIFICATION"
		
		"C_TRIG0_UNITS" "C_TRIG0_TRIGGER_IN_WIDTH" "C_TRIG0_UNIT_COUNTER_WIDTH" "C_TRIG0_UNIT_MATCH_TYPE"
		"C_TRIG1_UNITS" "C_TRIG1_TRIGGER_IN_WIDTH" "C_TRIG1_UNIT_COUNTER_WIDTH" "C_TRIG1_UNIT_MATCH_TYPE"
		"C_TRIG2_UNITS" "C_TRIG2_TRIGGER_IN_WIDTH" "C_TRIG2_UNIT_COUNTER_WIDTH" "C_TRIG2_UNIT_MATCH_TYPE"
		"C_TRIG3_UNITS" "C_TRIG3_TRIGGER_IN_WIDTH" "C_TRIG3_UNIT_COUNTER_WIDTH" "C_TRIG3_UNIT_MATCH_TYPE"
		"C_TRIG4_UNITS" "C_TRIG4_TRIGGER_IN_WIDTH" "C_TRIG4_UNIT_COUNTER_WIDTH" "C_TRIG4_UNIT_MATCH_TYPE"
		"C_TRIG5_UNITS" "C_TRIG5_TRIGGER_IN_WIDTH" "C_TRIG5_UNIT_COUNTER_WIDTH" "C_TRIG5_UNIT_MATCH_TYPE"
		"C_TRIG6_UNITS" "C_TRIG6_TRIGGER_IN_WIDTH" "C_TRIG6_UNIT_COUNTER_WIDTH" "C_TRIG6_UNIT_MATCH_TYPE"
		"C_TRIG7_UNITS" "C_TRIG7_TRIGGER_IN_WIDTH" "C_TRIG7_UNIT_COUNTER_WIDTH" "C_TRIG7_UNIT_MATCH_TYPE"
		"C_TRIG8_UNITS" "C_TRIG8_TRIGGER_IN_WIDTH" "C_TRIG8_UNIT_COUNTER_WIDTH" "C_TRIG8_UNIT_MATCH_TYPE"
		"C_TRIG9_UNITS" "C_TRIG9_TRIGGER_IN_WIDTH" "C_TRIG9_UNIT_COUNTER_WIDTH" "C_TRIG9_UNIT_MATCH_TYPE"
		"C_TRIG10_UNITS" "C_TRIG10_TRIGGER_IN_WIDTH" "C_TRIG10_UNIT_COUNTER_WIDTH" "C_TRIG10_UNIT_MATCH_TYPE"
		"C_TRIG11_UNITS" "C_TRIG11_TRIGGER_IN_WIDTH" "C_TRIG11_UNIT_COUNTER_WIDTH" "C_TRIG11_UNIT_MATCH_TYPE"
		"C_TRIG12_UNITS" "C_TRIG12_TRIGGER_IN_WIDTH" "C_TRIG12_UNIT_COUNTER_WIDTH" "C_TRIG12_UNIT_MATCH_TYPE"
		"C_TRIG13_UNITS" "C_TRIG13_TRIGGER_IN_WIDTH" "C_TRIG13_UNIT_COUNTER_WIDTH" "C_TRIG13_UNIT_MATCH_TYPE"
		"C_TRIG14_UNITS" "C_TRIG14_TRIGGER_IN_WIDTH" "C_TRIG14_UNIT_COUNTER_WIDTH" "C_TRIG14_UNIT_MATCH_TYPE"
		"C_TRIG15_UNITS" "C_TRIG15_TRIGGER_IN_WIDTH" "C_TRIG15_UNIT_COUNTER_WIDTH" "C_TRIG15_UNIT_MATCH_TYPE"
	}
	
	set param_table {}
	
	foreach name $param_names {
		lappend param_table $name
		lappend param_table [xget_value $handle "PARAMETER" $name]
	}
	lappend param_table IPNAME
	lappend param_table $ipname
	
	# put the PARAMETER name value pairs into an associative array 
	# for easy access Ex: set instance_name $params(INSTANCE)
	array set params $param_table
	
	if { $params(C_TRIG0_UNITS) == 0 &&
			 $params(C_TRIG1_UNITS) == 0 &&
			 $params(C_TRIG2_UNITS) == 0 &&
			 $params(C_TRIG3_UNITS) == 0 &&
			 $params(C_TRIG4_UNITS) == 0 &&
			 $params(C_TRIG5_UNITS) == 0 &&
			 $params(C_TRIG6_UNITS) == 0 &&
			 $params(C_TRIG7_UNITS) == 0 &&
			 $params(C_TRIG8_UNITS) == 0 &&
			 $params(C_TRIG9_UNITS) == 0 &&
			 $params(C_TRIG10_UNITS) == 0 &&
			 $params(C_TRIG11_UNITS) == 0 &&
			 $params(C_TRIG12_UNITS) == 0 &&
			 $params(C_TRIG13_UNITS) == 0 &&
			 $params(C_TRIG14_UNITS) == 0 &&
			 $params(C_TRIG15_UNITS) == 0 } {
		
		error "ERROR: None of the Chipscope ILA Trigger units (TRIG<n>_UNITS) are enabled. Please remove the ILA or enable atleast one of the trigger units"
	}
	
	# Chipscope User Settable Parameters
#	set data_sample_range [get_data_sample_range $params(C_FAMILY)]
#	set list_data_sample_range [split $data_sample_range]
#	if { [lsearch $list_data_sample_range $params(C_NUM_DATA_SAMPLES)] == -1 } {
#		error "Invalid value : $params(C_NUM_DATA_SAMPLES) for Parameter C_NUM_DATA_SAMPLES.
#Valid values are : $data_sample_range."
#	}
	
}

#***--------------------------------***-----------------------------------***
#
#			     SIMGEN_SYSLEVEL_UPDATE_PROC
#
#***--------------------------------***-----------------------------------***

proc ila_simulate { handle } {
	
}

#***--------------------------------***-----------------------------------***
#
#			     PLATGEN_SYSLEVEL_UPDATE_PROC
#
#***--------------------------------***-----------------------------------***

proc ila_generate { handle } {
	
	set ipname "chipscope_ila"
	
	#puts "**********************************************"
	#puts "**********************************************"
	
	set param_names { "INSTANCE" "HW_VER" "C_FAMILY"
		"C_DEVICE" "C_PACKAGE" "C_SPEEDGRADE"
		"C_NUM_DATA_SAMPLES" "C_ENABLE_TRIGGER_OUT"
		"C_DATA_SAME_AS_TRIGGER" "C_DATA_IN_WIDTH"
		"C_DISABLE_RPM" "C_DISABLE_SRL16S" "C_RISING_CLOCK_EDGE" 
		"C_MAX_SEQUENCER_LEVELS" "C_ENABLE_STORAGE_QUALIFICATION"
		
		"C_TRIG0_UNITS" "C_TRIG0_TRIGGER_IN_WIDTH" "C_TRIG0_UNIT_COUNTER_WIDTH" "C_TRIG0_UNIT_MATCH_TYPE"
		"C_TRIG1_UNITS" "C_TRIG1_TRIGGER_IN_WIDTH" "C_TRIG1_UNIT_COUNTER_WIDTH" "C_TRIG1_UNIT_MATCH_TYPE"
		"C_TRIG2_UNITS" "C_TRIG2_TRIGGER_IN_WIDTH" "C_TRIG2_UNIT_COUNTER_WIDTH" "C_TRIG2_UNIT_MATCH_TYPE"
		"C_TRIG3_UNITS" "C_TRIG3_TRIGGER_IN_WIDTH" "C_TRIG3_UNIT_COUNTER_WIDTH" "C_TRIG3_UNIT_MATCH_TYPE"
		"C_TRIG4_UNITS" "C_TRIG4_TRIGGER_IN_WIDTH" "C_TRIG4_UNIT_COUNTER_WIDTH" "C_TRIG4_UNIT_MATCH_TYPE"
		"C_TRIG5_UNITS" "C_TRIG5_TRIGGER_IN_WIDTH" "C_TRIG5_UNIT_COUNTER_WIDTH" "C_TRIG5_UNIT_MATCH_TYPE"
		"C_TRIG6_UNITS" "C_TRIG6_TRIGGER_IN_WIDTH" "C_TRIG6_UNIT_COUNTER_WIDTH" "C_TRIG6_UNIT_MATCH_TYPE"
		"C_TRIG7_UNITS" "C_TRIG7_TRIGGER_IN_WIDTH" "C_TRIG7_UNIT_COUNTER_WIDTH" "C_TRIG7_UNIT_MATCH_TYPE"
		"C_TRIG8_UNITS" "C_TRIG8_TRIGGER_IN_WIDTH" "C_TRIG8_UNIT_COUNTER_WIDTH" "C_TRIG8_UNIT_MATCH_TYPE"
		"C_TRIG9_UNITS" "C_TRIG9_TRIGGER_IN_WIDTH" "C_TRIG9_UNIT_COUNTER_WIDTH" "C_TRIG9_UNIT_MATCH_TYPE"
		"C_TRIG10_UNITS" "C_TRIG10_TRIGGER_IN_WIDTH" "C_TRIG10_UNIT_COUNTER_WIDTH" "C_TRIG10_UNIT_MATCH_TYPE"
		"C_TRIG11_UNITS" "C_TRIG11_TRIGGER_IN_WIDTH" "C_TRIG11_UNIT_COUNTER_WIDTH" "C_TRIG11_UNIT_MATCH_TYPE"
		"C_TRIG12_UNITS" "C_TRIG12_TRIGGER_IN_WIDTH" "C_TRIG12_UNIT_COUNTER_WIDTH" "C_TRIG12_UNIT_MATCH_TYPE"
		"C_TRIG13_UNITS" "C_TRIG13_TRIGGER_IN_WIDTH" "C_TRIG13_UNIT_COUNTER_WIDTH" "C_TRIG13_UNIT_MATCH_TYPE"
		"C_TRIG14_UNITS" "C_TRIG14_TRIGGER_IN_WIDTH" "C_TRIG14_UNIT_COUNTER_WIDTH" "C_TRIG14_UNIT_MATCH_TYPE"
		"C_TRIG15_UNITS" "C_TRIG15_TRIGGER_IN_WIDTH" "C_TRIG15_UNIT_COUNTER_WIDTH" "C_TRIG15_UNIT_MATCH_TYPE"
	}
	
	set param_table {}
	
	foreach name $param_names {
		lappend param_table $name
		lappend param_table [xget_value $handle "PARAMETER" $name]
	}
	lappend param_table IPNAME
	lappend param_table $ipname
	
	#puts "**********************************************"
	#puts "**********************************************"
	#puts "Param values are : $param_table"
	
	# put the PARAMETER name value pairs into an associative array 
	# for easy access Ex: set instance_name $params(INSTANCE)
	array set params $param_table
	
	# First Create the HDL wrapper to instantiate the Chipscope core
	# This intermediate wrapper is needed to match the port declaration
	# (no of ports is fixed) in the EDK Chipscope core and the port declaration
	# in the Chipscope core generated by the Chipscope Core Generator engine
	# Wrapper location:
	# <proj>/hdl/elaborate/<cs_core>/synhdl/vhdl/<cs_core>_wrap.vhd
	
	write_hdl_wrapper $handle $param_table
	
	if { $params(C_TRIG0_UNITS) == 0 &&
			 $params(C_TRIG1_UNITS) == 0 &&
			 $params(C_TRIG2_UNITS) == 0 &&
			 $params(C_TRIG3_UNITS) == 0 &&
			 $params(C_TRIG4_UNITS) == 0 &&
			 $params(C_TRIG5_UNITS) == 0 &&
			 $params(C_TRIG6_UNITS) == 0 &&
			 $params(C_TRIG7_UNITS) == 0 &&
			 $params(C_TRIG8_UNITS) == 0 &&
			 $params(C_TRIG9_UNITS) == 0 &&
			 $params(C_TRIG10_UNITS) == 0 &&
			 $params(C_TRIG11_UNITS) == 0 &&
			 $params(C_TRIG12_UNITS) == 0 &&
			 $params(C_TRIG13_UNITS) == 0 &&
			 $params(C_TRIG14_UNITS) == 0 &&
			 $params(C_TRIG15_UNITS) == 0 } {
		
		puts "WARNING: None of the Chipscope ILA Trigger units (TRIG<n>_UNITS) are enabled. This block will be trimmed away by MAP."
	} else {
		
		# Second, write out a Chipscope Core Generator xco file that matches
		# the parameters set by the user.
		
		set netlist_directory [file nativename [file join [pwd] implementation "$params(INSTANCE)_wrapper"]]  

		set xco_file [write_xco_file $param_table $netlist_directory]
		
		# Check if parameters are the same. If so, do not regenerate the core
		# Do this by comparing current .xco file and previous .xco.last file
		
		if { [file exists $xco_file.last] } {
	    if { [file size $xco_file] == [file size $xco_file.last] } {
				set argf_cur [open $xco_file]
				set argf_last [open $xco_file.last]
				while { ![eof $argf_cur] } {
					if { [gets $argf_cur] != [gets $argf_last] } {
						break
					}
				}
				if { [eof $argf_cur] } {
					puts "INFO: Chipscope core parameters not changed. Not regenerating core"
					close $argf_cur
					close $argf_last
					return
				}
				close $argf_cur
				close $argf_last
				file delete -force $xco_file.last
	    }
		}
    
		# Third, call the Chipscope Core Generator with the xco file to create
		# the core netlist in <proj>/implementation/CORE_INSTANCENAME/.
		# Platgen will run ngcbuild from this location to merge the netlists 
		# into a single NGC file. This helps in case of multiple instantiations
		# of the same core.
		
		set instance_name $params(INSTANCE)
		puts "Generating ChipScope core: $params(INSTANCE) ..."
		
    # Start with clean directory for Coregen
    if {[file exists $netlist_directory]} {
      file delete -force $netlist_directory  
    }
    file mkdir $netlist_directory  
	
		if { [catch {generate_chipscope_core $xco_file $instance_name $netlist_directory} errMsg] } {
			
	    error $errMsg
			
		}

	}
	
	# Augment the CDC file with signal name declarations and buses
	
	if { [catch {update_cdc_file $handle $param_table} err] } {
		puts "WARNING: Could not update CDC file:
              $err
           "
	}
	
	puts "*************************************************************"
	#puts "Successfully generated the ChipScope Core : $params(INSTANCE)"
	#puts "*************************************************************"
	
	# Keep a copy of the xco file for comparison next time.
	# This is the caching mechanism.
	file copy -force $xco_file $xco_file.last
	
	return
}


#proc get_devicefamily { c_family } {
	
#	switch $c_family {
#		"qvirtex"   { return "qpro virtex hi-rel" }
#		"qrvirtex"  { return "qpro virtex rad-hard" }
#		"qvirtexe"  { return "qpro virtexe military" }
#		"qvirtex2"  { return "qpro virtex2 military" }
#		"qrvirtex2" { return "qpro virtex2 rad tolerant" }
#		"spartan3adsp" { return "Spartan-3A DSP" }
#		default { return $c_family }
#	}
#}

# Return data sample range
# Virtex5         {     1k, 2k, 4k, 8k, 16k, 32k, 64k, 128k}, default 1k
# Earlier devices {512, 1k, 2k, 4k, 8k}, default 512

#proc get_data_sample_range { c_family } {
#	switch [string tolower $c_family] {
#		"virtex5"   { return "1024 2048 4096 8192 16384 32768 65536 131072" }
#		default     { return "512 1024 2048 4096 8192 16384" }
#	}
#}

proc write_xco_file { param_table netlist_directory} {
	
	# put the PARAMETER name value pairs into an associative array 
	# for easy access Ex: set instance_name $params(INSTANCE)
	array set params $param_table
	
	# Set initial values of some global variables
	set datawidth 0
	set CurTrigPort 1
	
#	set filename "implementation/$params(INSTANCE).xco"
	set filename [file join [pwd] "implementation" [format "%s.xco" $params(INSTANCE)]]
	set xcofile [open $filename "w"]
	
	puts $xcofile "#################################################################"
	puts $xcofile "## ChipScope ILA XCO file generated by chipscope_ila_v2_1_0.tcl #"
	puts $xcofile "#################################################################"
	
	puts $xcofile "NEWPROJECT $netlist_directory"
	puts $xcofile "SETPROJECT $netlist_directory"
	
	puts $xcofile "SET devicefamily = $params(C_FAMILY)"
	puts $xcofile "SET device = $params(C_DEVICE)"
	puts $xcofile "SET package = $params(C_PACKAGE)"
	puts $xcofile "SET speedgrade = $params(C_SPEEDGRADE)"
	
	puts $xcofile "SET simulationfiles = structural"
	puts $xcofile "SET vhdlsim = False"
	puts $xcofile "SET verilogsim = False"
	
	puts $xcofile "SELECT ILA_(ChipScope_Pro_-_Integrated_Logic_Analyzer) family Xilinx,_Inc. 1.05.a"
	
	puts $xcofile "CSET component_name = $params(INSTANCE)"
	
	puts $xcofile "CSET sample_data_depth = $params(C_NUM_DATA_SAMPLES)"
	
	puts -nonewline $xcofile "CSET data_same_as_trigger = "
	if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
		puts $xcofile "true"
	} else {
		puts $xcofile "false"
	}
	
	if { [create_ila_match_unit $xcofile $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG0_UNITS) $params(C_TRIG0_UNIT_MATCH_TYPE) $params(C_TRIG0_UNIT_COUNTER_WIDTH)  $params(C_TRIG0_TRIGGER_IN_WIDTH) "TRIG0 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG0_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG1_UNITS) $params(C_TRIG1_UNIT_MATCH_TYPE) $params(C_TRIG1_UNIT_COUNTER_WIDTH)  $params(C_TRIG1_TRIGGER_IN_WIDTH) "TRIG1 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG1_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG2_UNITS) $params(C_TRIG2_UNIT_MATCH_TYPE) $params(C_TRIG2_UNIT_COUNTER_WIDTH)  $params(C_TRIG2_TRIGGER_IN_WIDTH) "TRIG2 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG2_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG3_UNITS) $params(C_TRIG3_UNIT_MATCH_TYPE) $params(C_TRIG3_UNIT_COUNTER_WIDTH)  $params(C_TRIG3_TRIGGER_IN_WIDTH) "TRIG3 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG3_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG4_UNITS) $params(C_TRIG4_UNIT_MATCH_TYPE) $params(C_TRIG4_UNIT_COUNTER_WIDTH)  $params(C_TRIG4_TRIGGER_IN_WIDTH) "TRIG4 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG4_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG5_UNITS) $params(C_TRIG5_UNIT_MATCH_TYPE) $params(C_TRIG5_UNIT_COUNTER_WIDTH)  $params(C_TRIG5_TRIGGER_IN_WIDTH) "TRIG5 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG5_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG6_UNITS) $params(C_TRIG6_UNIT_MATCH_TYPE) $params(C_TRIG6_UNIT_COUNTER_WIDTH)  $params(C_TRIG6_TRIGGER_IN_WIDTH) "TRIG6 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG6_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG7_UNITS) $params(C_TRIG7_UNIT_MATCH_TYPE) $params(C_TRIG7_UNIT_COUNTER_WIDTH)  $params(C_TRIG7_TRIGGER_IN_WIDTH) "TRIG7 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG7_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG8_UNITS) $params(C_TRIG8_UNIT_MATCH_TYPE) $params(C_TRIG8_UNIT_COUNTER_WIDTH)  $params(C_TRIG8_TRIGGER_IN_WIDTH) "TRIG8 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG8_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG9_UNITS) $params(C_TRIG9_UNIT_MATCH_TYPE) $params(C_TRIG9_UNIT_COUNTER_WIDTH)  $params(C_TRIG9_TRIGGER_IN_WIDTH) "TRIG9 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG9_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG10_UNITS) $params(C_TRIG10_UNIT_MATCH_TYPE) $params(C_TRIG10_UNIT_COUNTER_WIDTH)  $params(C_TRIG10_TRIGGER_IN_WIDTH) "TRIG10 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG10_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG11_UNITS) $params(C_TRIG11_UNIT_MATCH_TYPE) $params(C_TRIG11_UNIT_COUNTER_WIDTH)  $params(C_TRIG11_TRIGGER_IN_WIDTH) "TRIG11 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG11_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG12_UNITS) $params(C_TRIG12_UNIT_MATCH_TYPE) $params(C_TRIG12_UNIT_COUNTER_WIDTH)  $params(C_TRIG12_TRIGGER_IN_WIDTH) "TRIG12 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG12_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG13_UNITS) $params(C_TRIG13_UNIT_MATCH_TYPE) $params(C_TRIG13_UNIT_COUNTER_WIDTH)  $params(C_TRIG13_TRIGGER_IN_WIDTH) "TRIG13 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG13_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG14_UNITS) $params(C_TRIG14_UNIT_MATCH_TYPE) $params(C_TRIG14_UNIT_COUNTER_WIDTH)  $params(C_TRIG14_TRIGGER_IN_WIDTH) "TRIG14 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG14_TRIGGER_IN_WIDTH)
		}
	}
	
	if { [create_ila_match_unit $xcofile  $params(C_DATA_SAME_AS_TRIGGER) $params(C_TRIG15_UNITS) $params(C_TRIG15_UNIT_MATCH_TYPE) $params(C_TRIG15_UNIT_COUNTER_WIDTH)  $params(C_TRIG15_TRIGGER_IN_WIDTH) "TRIG15 Trigger Unit" $CurTrigPort] } {
		incr CurTrigPort 1
		if { $params(C_DATA_SAME_AS_TRIGGER) != 0 } {
	    incr datawidth $params(C_TRIG15_TRIGGER_IN_WIDTH)
		}
	}
	
	# Special - enable trigger output
	puts -nonewline $xcofile "CSET enable_trigger_output_port = "
	if { $params(C_ENABLE_TRIGGER_OUT) == 1 } {
		puts $xcofile "true"
	} else {
		puts $xcofile "false"
	}
	
	# Special - disable RPM (new parameter to disable RPM)
	puts -nonewline $xcofile "CSET use_rpms = "
	if { $params(C_DISABLE_RPM) == 1 } {
		puts $xcofile "false"
	} else {
		puts $xcofile "true"
	}
	
	# Special - Rising/Falling clock edge
	puts -nonewline $xcofile "CSET sample_on = "
	if { $params(C_RISING_CLOCK_EDGE) == 0 } {
		puts $xcofile "Falling"
	} else {
		puts $xcofile "Rising"
	}
	
	# Special - Trigger Sequencer
	puts -nonewline $xcofile "CSET max_sequence_levels = "
  puts $xcofile "$params(C_MAX_SEQUENCER_LEVELS)"
	
	# Special - Storage Qualification
	puts -nonewline $xcofile "CSET enable_storage_qualification = "
	if { $params(C_ENABLE_STORAGE_QUALIFICATION) == 0 } {
		puts $xcofile "false"
	} else {
		puts $xcofile "true"
	}
	
	puts $xcofile "CSET number_of_trigger_ports = [expr $CurTrigPort - 1]"
	
	puts -nonewline $xcofile "CSET data_port_width = "
	if { $params(C_DATA_SAME_AS_TRIGGER) == 0 } {
		puts $xcofile "$params(C_DATA_IN_WIDTH)"
	} else {
		puts $xcofile "0"
	}
	
	puts $xcofile "GENERATE"
	
	close $xcofile
	
	return $filename
}

#
# The SRL type is dependent on the Architecture
# 
#proc get_srl16_type {fpga_family disable_srl16 } {
	## Default value - dont use SRLs
	
#	set srl16_type 0
	
#	switch [string tolower $fpga_family] {
		
#		"qvirtex"                   -
#		"qpro virtex hi-rel"        -
#		"qrvirtex"                  -
#		"qpro virtex rad-hard"      -
#		"qvirtexe"                  -
#		"qpro virtexe military"     { set srl16_type 2 }
		
#		"spartan2"  -
#		"spartan2e" -
#		"virtex"    -
#		"virtexe"  { set srl16_type 1 }
		
#		"qvirtex2"                   -
#		"qpro virtex2 military"      -
#		"qrvirtex2"                  -
#		"qpro virtex2 rad tolerant"  -
#		"spartan3"  -
#		"spartan3e" -
#		"spartan3a" -
#		"spartan3adsp" -
#		"virtex2"   -
#		"virtex2p"  -
#		"virtex4"  -
#		"virtex5"   {
#	    if { $disable_srl16 != 0 }  { 
#				set srl16_type 0
#	    } else  { 
#				set srl16_type 2 
#	    }
#		}
		
#		default {
#	    puts "Warning: Unknown FPGA architecture. Disabling SRL16 instantiation"
#	    set srl16_type 0
#		}
#	}
	
#	return $srl16_type
#}


#
# For most match units, users can choose the type of match units :
#    basic, basic with edges, extended, extended with edges, etc
# These text values have to be convert
# 

proc get_match_type { typestr match_unit } {
	set type 0
	
	switch $typestr {
		"7"     -
		"range with edges" -
		"range_with_edges" { return "range_with_edges" }
		
		"6"     -
		"range" { return "range" }
		
		"5"     -
		"extended with edges" -
		"extended_with_edges" { return "extended_with_edges" }
		
		"4"     -
		"extended" { return "extended" }
		
		"1"     -
		"basic with edges" -
		"basic_with_edges" { return "basic_with_edges" }
		
		"0"     -
		"basic"   { return "basic" }
		
		default {
	    error "For $match_unit Match Unit Invalid Match type : $typestr. Valid values : basic, basic with edges, extended, extended with edges, range, range with edges."
		}
	}
}


proc create_ila_match_unit { xcofile data_same_as_trigger num_units match_type
														 counter_width signal_width unit_name 
														 CurTrigPort } {
	
	if { $num_units > 0 } {
		#if { $data_same_as_trigger != 0 } {
		#    # Enable Data and Trig for nth Unit
		#    puts $xcofile "-enabledatatrig$CurTrigPort"
		#}   
		
		# Enable/Disable Data and Trig for nth Unit
		puts -nonewline $xcofile "CSET exclude_from_data_storage_$CurTrigPort = "
		if { $data_same_as_trigger == 0 } {
	    puts $xcofile "true"
		} else {
	    puts $xcofile "false"
		}
		
		#  Width of Signals for triggering
		#puts $xcofile "-trigportwidth$CurTrigPort=$signal_width" 
		puts $xcofile "CSET trigger_port_width_$CurTrigPort = $signal_width"
		
		# Number of units for the current trigger port
		puts $xcofile "CSET match_units_$CurTrigPort = $num_units"
		
		# Match type for the current trigger port
		puts $xcofile "CSET match_type_$CurTrigPort = [get_match_type $match_type $unit_name]"
		
		puts -nonewline $xcofile "CSET counter_width_$CurTrigPort = "
		if { $counter_width > 0 } {
	    puts $xcofile $counter_width
		} else {
	    puts $xcofile "Disabled"
		}
		
		return 1
	}
	return 0
}

#***--------------------------------***-----------------------------------***
# Utility process to call a command and pipe it's output to screen.
# Used instead of Tcl's exec
proc execpipe {COMMAND} {

  if { [catch {open "| $COMMAND 2>@stdout"} FILEHANDLE] } {
    return "Can't open pipe for '$COMMAND'"
  }

  set PIPE $FILEHANDLE
  fconfigure $PIPE -buffering none
  
  set OUTPUT ""
  
  while { [gets $PIPE DATA] >= 0 } {
    puts $DATA
    append OUTPUT $DATA "\n"
  }
  
  if { [catch {close $PIPE} ERRORMSG] } {
    
    if { [string compare "$ERRORMSG" "child process exited abnormally"] == 0 } {
      # this error means there was nothing on stderr (which makes sense) and
      # there was a non-zero exit code - this is OK as we intentionally send
      # stderr to stdout, so we just do nothing here (and return the output)
    } else {
      return "Error '$ERRORMSG' on closing pipe for '$COMMAND'"
    }

  }

  regsub -all -- "\n$" $OUTPUT "" STRIPPED_STRING
  return "$STRIPPED_STRING"

}


#***--------------------------------***-----------------------------------***
proc generate_chipscope_core { xco_file instance_name netlist_directory } {
	global tcl_platform env tcl_library
	
	switch -glob $tcl_platform(os) {
		"Windows*" {
	    if { [auto_execok coregen] == ""} {
				error "Unable to find the ChipScope Core Generator \"coregen.exe\" in the path. Please ensure that a valid ISE Installation exists."
			}
			set coregen_cmd "coregen.exe"
		}
		
		"Linux" {
	    if { [auto_execok coregen] == ""} {
				error "Unable to find the ChipScope Core Generator \"coregen\" in the path. Please ensure that a valid ISE Installation exists."
			}
			set coregen_cmd "coregen"
		}
		
		default {
	    error "
****************************************************************************
Instantiation of ChipScope cores is not supported on $tcl_platform(os)
The ChipScope Core Generator engine \"coregen\" is not available on $tcl_platform(os).
To fix the problem remove this core from the MHS file
OR use Windows, Linux or Solaris platform to instantiate this core
****************************************************************************" "Platform Not Supported" "CS_UNAVAILABLE"
		}
	}
	
  # remove ngc (about to generate the file through coregen) 
	set ngc_filename [file join $netlist_directory [format "%s.ngc" $instance_name]]
	file delete -force $ngc_filename

  # prepare to run the command. it will be run from $netlist_directory.
  # this is done so the log file is created in the netlist directory.
  set current_pwd [pwd]
  cd $netlist_directory

  # run the command
  puts -nonewline "ChipScope Core Generator command: "
  puts "$coregen_cmd -b $xco_file"
  execpipe "$coregen_cmd -b $xco_file"
  cd $current_pwd

  # check for output ngc file
  if {![file exists $ngc_filename]} {
    error "ERROR: Coregen did not generate file \"$ngc_filename\"" ""
  }

##############
  set srcNcfFile [file join $netlist_directory [format "%s.ncf" $instance_name]]

  set wrapper_UcfFile $instance_name
  append wrapper_UcfFile "_wrapper.ucf"

  set tgtUcfFile [file join $netlist_directory $wrapper_UcfFile]

  file rename -- $srcNcfFile $tgtUcfFile
##############

	return
}

#***--------------------------------***-----------------------------------***
proc write_hdl_wrapper { handle param_table } {
	
	# put the PARAMETER name value pairs into an associative array 
	# for easy access Ex: set instance_name $params(INSTANCE)
	array set params $param_table
	
	set hdllib $params(INSTANCE)
	set hw_ver [string map {. _} [xget_hw_parameter_value $handle "HW_VER"]]
	if { [string length $hw_ver] != 0 } {
		append hdllib "_v" $hw_ver
	}
	
	set hdldir "hdl/elaborate/$hdllib/synhdl/vhdl/"
  if {![file exists $hdldir]} {
    file mkdir $hdldir
  }
	puts "Created elaborate directory $hdldir"
	
	set filename "$params(INSTANCE).vhd"
	set filepath [file join $hdldir/$filename]
	
	set hdlfile [open $filepath "w"]
	
	# Update PAO
	# xadd_hw_hdl_srcfile <ipinst_handle> <fileuse> <filename> <hdllang>
	xadd_hw_hdl_srcfile $handle "synlib" $filename "VHDL"
	
	puts $hdlfile "-- ChipScope ILA Wrapper HDL file generated by chipscope_ila's TCL Generator"
	puts $hdlfile "

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
library Unisim;
use Unisim.all;

entity chipscope_ila is
  generic (
    C_FAMILY           : string   := \"$params(C_FAMILY)\";
    C_DEVICE           : string   := \"$params(C_DEVICE)\";
    C_PACKAGE          : string   := \"$params(C_PACKAGE)\";
    C_SPEEDGRADE       : string   := \"$params(C_SPEEDGRADE)\";
    C_NUM_DATA_SAMPLES : integer  := $params(C_NUM_DATA_SAMPLES);
    C_DATA_SAME_AS_TRIGGER : integer := $params(C_DATA_SAME_AS_TRIGGER);
    C_DATA_IN_WIDTH : integer := $params(C_DATA_IN_WIDTH);
    C_ENABLE_TRIGGER_OUT : integer := $params(C_ENABLE_TRIGGER_OUT);
    C_DISABLE_RPM : integer := $params(C_DISABLE_RPM);

    C_TRIG0_UNITS : integer := $params(C_TRIG0_UNITS);
    C_TRIG0_TRIGGER_IN_WIDTH : integer := $params(C_TRIG0_TRIGGER_IN_WIDTH);
    C_TRIG0_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG0_UNIT_COUNTER_WIDTH);
    C_TRIG0_UNIT_MATCH_TYPE : string := \"$params(C_TRIG0_UNIT_MATCH_TYPE)\";

    C_TRIG1_UNITS : integer := $params(C_TRIG1_UNITS);
    C_TRIG1_TRIGGER_IN_WIDTH : integer := $params(C_TRIG1_TRIGGER_IN_WIDTH);
    C_TRIG1_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG1_UNIT_COUNTER_WIDTH);
    C_TRIG1_UNIT_MATCH_TYPE : string := \"$params(C_TRIG1_UNIT_MATCH_TYPE)\";

    C_TRIG2_UNITS : integer := $params(C_TRIG2_UNITS);
    C_TRIG2_TRIGGER_IN_WIDTH : integer := $params(C_TRIG2_TRIGGER_IN_WIDTH);
    C_TRIG2_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG2_UNIT_COUNTER_WIDTH);
    C_TRIG2_UNIT_MATCH_TYPE : string := \"$params(C_TRIG2_UNIT_MATCH_TYPE)\";

    C_TRIG3_UNITS : integer := $params(C_TRIG3_UNITS);
    C_TRIG3_TRIGGER_IN_WIDTH : integer := $params(C_TRIG3_TRIGGER_IN_WIDTH);
    C_TRIG3_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG3_UNIT_COUNTER_WIDTH);
    C_TRIG3_UNIT_MATCH_TYPE : string := \"$params(C_TRIG3_UNIT_MATCH_TYPE)\";

    C_TRIG4_UNITS : integer := $params(C_TRIG4_UNITS);
    C_TRIG4_TRIGGER_IN_WIDTH : integer := $params(C_TRIG4_TRIGGER_IN_WIDTH);
    C_TRIG4_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG4_UNIT_COUNTER_WIDTH);
    C_TRIG4_UNIT_MATCH_TYPE : string := \"$params(C_TRIG4_UNIT_MATCH_TYPE)\";

    C_TRIG5_UNITS : integer := $params(C_TRIG5_UNITS);
    C_TRIG5_TRIGGER_IN_WIDTH : integer := $params(C_TRIG5_TRIGGER_IN_WIDTH);
    C_TRIG5_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG5_UNIT_COUNTER_WIDTH);
    C_TRIG5_UNIT_MATCH_TYPE : string := \"$params(C_TRIG5_UNIT_MATCH_TYPE)\";

    C_TRIG6_UNITS : integer := $params(C_TRIG6_UNITS);
    C_TRIG6_TRIGGER_IN_WIDTH : integer := $params(C_TRIG6_TRIGGER_IN_WIDTH);
    C_TRIG6_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG6_UNIT_COUNTER_WIDTH);
    C_TRIG6_UNIT_MATCH_TYPE : string := \"$params(C_TRIG6_UNIT_MATCH_TYPE)\";

    C_TRIG7_UNITS : integer := $params(C_TRIG7_UNITS);
    C_TRIG7_TRIGGER_IN_WIDTH : integer := $params(C_TRIG7_TRIGGER_IN_WIDTH);
    C_TRIG7_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG7_UNIT_COUNTER_WIDTH);
    C_TRIG7_UNIT_MATCH_TYPE : string := \"$params(C_TRIG7_UNIT_MATCH_TYPE)\";

    C_TRIG8_UNITS : integer := $params(C_TRIG8_UNITS);
    C_TRIG8_TRIGGER_IN_WIDTH : integer := $params(C_TRIG8_TRIGGER_IN_WIDTH);
    C_TRIG8_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG8_UNIT_COUNTER_WIDTH);
    C_TRIG8_UNIT_MATCH_TYPE : string := \"$params(C_TRIG8_UNIT_MATCH_TYPE)\";

    C_TRIG9_UNITS : integer := $params(C_TRIG9_UNITS);
    C_TRIG9_TRIGGER_IN_WIDTH : integer := $params(C_TRIG9_TRIGGER_IN_WIDTH);
    C_TRIG9_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG9_UNIT_COUNTER_WIDTH);
    C_TRIG9_UNIT_MATCH_TYPE : string := \"$params(C_TRIG9_UNIT_MATCH_TYPE)\";

    C_TRIG10_UNITS : integer := $params(C_TRIG10_UNITS);
    C_TRIG10_TRIGGER_IN_WIDTH : integer := $params(C_TRIG10_TRIGGER_IN_WIDTH);
    C_TRIG10_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG10_UNIT_COUNTER_WIDTH);
    C_TRIG10_UNIT_MATCH_TYPE : string := \"$params(C_TRIG10_UNIT_MATCH_TYPE)\";

    C_TRIG11_UNITS : integer := $params(C_TRIG11_UNITS);
    C_TRIG11_TRIGGER_IN_WIDTH : integer := $params(C_TRIG11_TRIGGER_IN_WIDTH);
    C_TRIG11_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG11_UNIT_COUNTER_WIDTH);
    C_TRIG11_UNIT_MATCH_TYPE : string := \"$params(C_TRIG11_UNIT_MATCH_TYPE)\";

    C_TRIG12_UNITS : integer := $params(C_TRIG12_UNITS);
    C_TRIG12_TRIGGER_IN_WIDTH : integer := $params(C_TRIG12_TRIGGER_IN_WIDTH);
    C_TRIG12_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG12_UNIT_COUNTER_WIDTH);
    C_TRIG12_UNIT_MATCH_TYPE : string := \"$params(C_TRIG12_UNIT_MATCH_TYPE)\";

    C_TRIG13_UNITS : integer := $params(C_TRIG13_UNITS);
    C_TRIG13_TRIGGER_IN_WIDTH : integer := $params(C_TRIG13_TRIGGER_IN_WIDTH);
    C_TRIG13_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG13_UNIT_COUNTER_WIDTH);
    C_TRIG13_UNIT_MATCH_TYPE : string := \"$params(C_TRIG13_UNIT_MATCH_TYPE)\";

    C_TRIG14_UNITS : integer := $params(C_TRIG14_UNITS);
    C_TRIG14_TRIGGER_IN_WIDTH : integer := $params(C_TRIG14_TRIGGER_IN_WIDTH);
    C_TRIG14_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG14_UNIT_COUNTER_WIDTH);
    C_TRIG14_UNIT_MATCH_TYPE : string := \"$params(C_TRIG14_UNIT_MATCH_TYPE)\";

    C_TRIG15_UNITS : integer := $params(C_TRIG15_UNITS);
    C_TRIG15_TRIGGER_IN_WIDTH : integer := $params(C_TRIG15_TRIGGER_IN_WIDTH);
    C_TRIG15_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG15_UNIT_COUNTER_WIDTH);
    C_TRIG15_UNIT_MATCH_TYPE : string := \"$params(C_TRIG15_UNIT_MATCH_TYPE)\"

    );  
  port (

    CHIPSCOPE_ILA_CONTROL : in std_logic_vector(35 downto 0);
    CLK : in std_logic;
    DATA : in std_logic_vector(C_DATA_IN_WIDTH-1 downto 0);
    TRIG0 : in std_logic_vector(C_TRIG0_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG1 : in std_logic_vector(C_TRIG1_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG2 : in std_logic_vector(C_TRIG2_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG3 : in std_logic_vector(C_TRIG3_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG4 : in std_logic_vector(C_TRIG4_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG5 : in std_logic_vector(C_TRIG5_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG6 : in std_logic_vector(C_TRIG6_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG7 : in std_logic_vector(C_TRIG7_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG8 : in std_logic_vector(C_TRIG8_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG9 : in std_logic_vector(C_TRIG9_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG10 : in std_logic_vector(C_TRIG10_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG11 : in std_logic_vector(C_TRIG11_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG12 : in std_logic_vector(C_TRIG12_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG13 : in std_logic_vector(C_TRIG13_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG14 : in std_logic_vector(C_TRIG14_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG15 : in std_logic_vector(C_TRIG15_TRIGGER_IN_WIDTH-1 downto 0);
    TRIG_OUT : out std_logic
    );
end entity chipscope_ila;

-----------------------------------------------------------------------------
-- Architecture section
-----------------------------------------------------------------------------

architecture imp of chipscope_ila is
    
-----------------------------------------------------------------------------
-- Component declarations
-----------------------------------------------------------------------------
  -------------------------------------------------------------------
  --
  --  ILA core component declaration
  -------------------------------------------------------------------
  component $params(INSTANCE) is

    generic ( 
    C_FAMILY           : string   := \"$params(C_FAMILY)\";
    C_NUM_DATA_SAMPLES : integer  := $params(C_NUM_DATA_SAMPLES);
    C_DATA_SAME_AS_TRIGGER : integer := $params(C_DATA_SAME_AS_TRIGGER);
    C_DATA_IN_WIDTH : integer := $params(C_DATA_IN_WIDTH);
    C_ENABLE_TRIGGER_OUT : integer := $params(C_ENABLE_TRIGGER_OUT);
    C_DISABLE_RPM : integer := $params(C_DISABLE_RPM);

    C_TRIG0_UNITS : integer := $params(C_TRIG0_UNITS);
    C_TRIG0_TRIGGER_IN_WIDTH : integer := $params(C_TRIG0_TRIGGER_IN_WIDTH);
    C_TRIG0_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG0_UNIT_COUNTER_WIDTH);
    C_TRIG0_UNIT_MATCH_TYPE : string := \"$params(C_TRIG0_UNIT_MATCH_TYPE)\";

    C_TRIG1_UNITS : integer := $params(C_TRIG1_UNITS);
    C_TRIG1_TRIGGER_IN_WIDTH : integer := $params(C_TRIG1_TRIGGER_IN_WIDTH);
    C_TRIG1_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG1_UNIT_COUNTER_WIDTH);
    C_TRIG1_UNIT_MATCH_TYPE : string := \"$params(C_TRIG1_UNIT_MATCH_TYPE)\";

    C_TRIG2_UNITS : integer := $params(C_TRIG2_UNITS);
    C_TRIG2_TRIGGER_IN_WIDTH : integer := $params(C_TRIG2_TRIGGER_IN_WIDTH);
    C_TRIG2_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG2_UNIT_COUNTER_WIDTH);
    C_TRIG2_UNIT_MATCH_TYPE : string := \"$params(C_TRIG2_UNIT_MATCH_TYPE)\";

    C_TRIG3_UNITS : integer := $params(C_TRIG3_UNITS);
    C_TRIG3_TRIGGER_IN_WIDTH : integer := $params(C_TRIG3_TRIGGER_IN_WIDTH);
    C_TRIG3_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG3_UNIT_COUNTER_WIDTH);
    C_TRIG3_UNIT_MATCH_TYPE : string := \"$params(C_TRIG3_UNIT_MATCH_TYPE)\";

    C_TRIG4_UNITS : integer := $params(C_TRIG4_UNITS);
    C_TRIG4_TRIGGER_IN_WIDTH : integer := $params(C_TRIG4_TRIGGER_IN_WIDTH);
    C_TRIG4_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG4_UNIT_COUNTER_WIDTH);
    C_TRIG4_UNIT_MATCH_TYPE : string := \"$params(C_TRIG4_UNIT_MATCH_TYPE)\";

    C_TRIG5_UNITS : integer := $params(C_TRIG5_UNITS);
    C_TRIG5_TRIGGER_IN_WIDTH : integer := $params(C_TRIG5_TRIGGER_IN_WIDTH);
    C_TRIG5_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG5_UNIT_COUNTER_WIDTH);
    C_TRIG5_UNIT_MATCH_TYPE : string := \"$params(C_TRIG5_UNIT_MATCH_TYPE)\";

    C_TRIG6_UNITS : integer := $params(C_TRIG6_UNITS);
    C_TRIG6_TRIGGER_IN_WIDTH : integer := $params(C_TRIG6_TRIGGER_IN_WIDTH);
    C_TRIG6_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG6_UNIT_COUNTER_WIDTH);
    C_TRIG6_UNIT_MATCH_TYPE : string := \"$params(C_TRIG6_UNIT_MATCH_TYPE)\";

    C_TRIG7_UNITS : integer := $params(C_TRIG7_UNITS);
    C_TRIG7_TRIGGER_IN_WIDTH : integer := $params(C_TRIG7_TRIGGER_IN_WIDTH);
    C_TRIG7_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG7_UNIT_COUNTER_WIDTH);
    C_TRIG7_UNIT_MATCH_TYPE : string := \"$params(C_TRIG7_UNIT_MATCH_TYPE)\";

    C_TRIG8_UNITS : integer := $params(C_TRIG8_UNITS);
    C_TRIG8_TRIGGER_IN_WIDTH : integer := $params(C_TRIG8_TRIGGER_IN_WIDTH);
    C_TRIG8_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG8_UNIT_COUNTER_WIDTH);
    C_TRIG8_UNIT_MATCH_TYPE : string := \"$params(C_TRIG8_UNIT_MATCH_TYPE)\";

    C_TRIG9_UNITS : integer := $params(C_TRIG9_UNITS);
    C_TRIG9_TRIGGER_IN_WIDTH : integer := $params(C_TRIG9_TRIGGER_IN_WIDTH);
    C_TRIG9_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG9_UNIT_COUNTER_WIDTH);
    C_TRIG9_UNIT_MATCH_TYPE : string := \"$params(C_TRIG9_UNIT_MATCH_TYPE)\";

    C_TRIG10_UNITS : integer := $params(C_TRIG10_UNITS);
    C_TRIG10_TRIGGER_IN_WIDTH : integer := $params(C_TRIG10_TRIGGER_IN_WIDTH);
    C_TRIG10_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG10_UNIT_COUNTER_WIDTH);
    C_TRIG10_UNIT_MATCH_TYPE : string := \"$params(C_TRIG10_UNIT_MATCH_TYPE)\";

    C_TRIG11_UNITS : integer := $params(C_TRIG11_UNITS);
    C_TRIG11_TRIGGER_IN_WIDTH : integer := $params(C_TRIG11_TRIGGER_IN_WIDTH);
    C_TRIG11_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG11_UNIT_COUNTER_WIDTH);
    C_TRIG11_UNIT_MATCH_TYPE : string := \"$params(C_TRIG11_UNIT_MATCH_TYPE)\";

    C_TRIG12_UNITS : integer := $params(C_TRIG12_UNITS);
    C_TRIG12_TRIGGER_IN_WIDTH : integer := $params(C_TRIG12_TRIGGER_IN_WIDTH);
    C_TRIG12_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG12_UNIT_COUNTER_WIDTH);
    C_TRIG12_UNIT_MATCH_TYPE : string := \"$params(C_TRIG12_UNIT_MATCH_TYPE)\";

    C_TRIG13_UNITS : integer := $params(C_TRIG13_UNITS);
    C_TRIG13_TRIGGER_IN_WIDTH : integer := $params(C_TRIG13_TRIGGER_IN_WIDTH);
    C_TRIG13_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG13_UNIT_COUNTER_WIDTH);
    C_TRIG13_UNIT_MATCH_TYPE : string := \"$params(C_TRIG13_UNIT_MATCH_TYPE)\";

    C_TRIG14_UNITS : integer := $params(C_TRIG14_UNITS);
    C_TRIG14_TRIGGER_IN_WIDTH : integer := $params(C_TRIG14_TRIGGER_IN_WIDTH);
    C_TRIG14_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG14_UNIT_COUNTER_WIDTH);
    C_TRIG14_UNIT_MATCH_TYPE : string := \"$params(C_TRIG14_UNIT_MATCH_TYPE)\";

    C_TRIG15_UNITS : integer := $params(C_TRIG15_UNITS);
    C_TRIG15_TRIGGER_IN_WIDTH : integer := $params(C_TRIG15_TRIGGER_IN_WIDTH);
    C_TRIG15_UNIT_COUNTER_WIDTH : integer := $params(C_TRIG15_UNIT_COUNTER_WIDTH);
    C_TRIG15_UNIT_MATCH_TYPE : string := \"$params(C_TRIG15_UNIT_MATCH_TYPE)\"
    ) ;
    port (
"
	if { $params(C_ENABLE_TRIGGER_OUT) != 0 } {
		puts $hdlfile "      trig_out      : out   std_logic;"
	}
	if { $params(C_TRIG0_UNITS) > 0 } {
		puts $hdlfile "      trig0       : in    std_logic_vector(C_TRIG0_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG1_UNITS) > 0 } {
		puts $hdlfile "      trig1       : in    std_logic_vector(C_TRIG1_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG2_UNITS) > 0 } {
		puts $hdlfile "      trig2       : in    std_logic_vector(C_TRIG2_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG3_UNITS) > 0 } {
		puts $hdlfile "      trig3       : in    std_logic_vector(C_TRIG3_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG4_UNITS) > 0 } {
		puts $hdlfile "      trig4       : in    std_logic_vector(C_TRIG4_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG5_UNITS) > 0 } {
		puts $hdlfile "      trig5       : in    std_logic_vector(C_TRIG5_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG6_UNITS) > 0 } {
		puts $hdlfile "      trig6       : in    std_logic_vector(C_TRIG6_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG7_UNITS) > 0 } {
		puts $hdlfile "      trig7       : in    std_logic_vector(C_TRIG7_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG8_UNITS) > 0 } {
		puts $hdlfile "      trig8       : in    std_logic_vector(C_TRIG8_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG9_UNITS) > 0 } {
		puts $hdlfile "      trig9       : in    std_logic_vector(C_TRIG9_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG10_UNITS) > 0 } {
		puts $hdlfile "      trig10       : in    std_logic_vector(C_TRIG10_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG11_UNITS) > 0 } {
		puts $hdlfile "      trig11       : in    std_logic_vector(C_TRIG11_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG12_UNITS) > 0 } {
		puts $hdlfile "      trig12       : in    std_logic_vector(C_TRIG12_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG13_UNITS) > 0 } {
		puts $hdlfile "      trig13       : in    std_logic_vector(C_TRIG13_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG14_UNITS) > 0 } {
		puts $hdlfile "      trig14       : in    std_logic_vector(C_TRIG14_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_TRIG15_UNITS) > 0 } {
		puts $hdlfile "      trig15       : in    std_logic_vector(C_TRIG15_TRIGGER_IN_WIDTH-1 downto 0);"
	}
	if { $params(C_DATA_SAME_AS_TRIGGER) == 0 } {
		puts $hdlfile "      data           : in    std_logic_vector(C_DATA_IN_WIDTH-1 downto 0);"
	}
	puts $hdlfile "
      clk           : in    std_logic;
      control       : in    std_logic_vector(35 downto 0)
    );
  end component $params(INSTANCE);
   

-----------------------------------------------------------------------------
-- Begin architecture
-----------------------------------------------------------------------------

begin  -- architecture imp

"
	
	#
	# If NO Trigger units are enabled, do not instantiate anything
	# 
	
	if { $params(C_TRIG0_UNITS) == 0 &&
			 $params(C_TRIG1_UNITS) == 0 &&
			 $params(C_TRIG2_UNITS) == 0 &&
			 $params(C_TRIG3_UNITS) == 0 &&
			 $params(C_TRIG4_UNITS) == 0 &&
			 $params(C_TRIG5_UNITS) == 0 &&
			 $params(C_TRIG6_UNITS) == 0 &&
			 $params(C_TRIG7_UNITS) == 0 &&
			 $params(C_TRIG8_UNITS) == 0 &&
			 $params(C_TRIG9_UNITS) == 0 &&
			 $params(C_TRIG10_UNITS) == 0 &&
			 $params(C_TRIG11_UNITS) == 0 &&
			 $params(C_TRIG12_UNITS) == 0 &&
			 $params(C_TRIG13_UNITS) == 0 &&
			 $params(C_TRIG14_UNITS) == 0 &&
			 $params(C_TRIG15_UNITS) == 0 } {
		
		puts $hdlfile "
-- No Trigger units were enabled, hence nothing is instantiated and the only output is grounded
     trig_out <= '0';

end architecture imp;
"
	} else {
		
		
		puts $hdlfile "
  -----------------------------------------------------------------------------
  -- ILA instantiation
  -----------------------------------------------------------------------------
  i_$params(INSTANCE) : $params(INSTANCE)
    generic map (
    C_FAMILY               => C_FAMILY,
    C_NUM_DATA_SAMPLES     => C_NUM_DATA_SAMPLES,
    C_DATA_SAME_AS_TRIGGER => C_DATA_SAME_AS_TRIGGER,
    C_DATA_IN_WIDTH        => C_DATA_IN_WIDTH,
    C_ENABLE_TRIGGER_OUT   => C_ENABLE_TRIGGER_OUT,
    C_DISABLE_RPM          => C_DISABLE_RPM,

    C_TRIG0_UNITS              => C_TRIG0_UNITS,
    C_TRIG0_TRIGGER_IN_WIDTH   => C_TRIG0_TRIGGER_IN_WIDTH,
    C_TRIG0_UNIT_COUNTER_WIDTH => C_TRIG0_UNIT_COUNTER_WIDTH,
    C_TRIG0_UNIT_MATCH_TYPE    => C_TRIG0_UNIT_MATCH_TYPE,

    C_TRIG1_UNITS              =>     C_TRIG1_UNITS,
    C_TRIG1_TRIGGER_IN_WIDTH   =>     C_TRIG1_TRIGGER_IN_WIDTH,
    C_TRIG1_UNIT_COUNTER_WIDTH =>     C_TRIG1_UNIT_COUNTER_WIDTH,
    C_TRIG1_UNIT_MATCH_TYPE    =>     C_TRIG1_UNIT_MATCH_TYPE,

    C_TRIG2_UNITS              =>     C_TRIG2_UNITS,
    C_TRIG2_TRIGGER_IN_WIDTH   =>     C_TRIG2_TRIGGER_IN_WIDTH,
    C_TRIG2_UNIT_COUNTER_WIDTH =>     C_TRIG2_UNIT_COUNTER_WIDTH,
    C_TRIG2_UNIT_MATCH_TYPE    =>     C_TRIG2_UNIT_MATCH_TYPE,

    C_TRIG3_UNITS              =>     C_TRIG3_UNITS,
    C_TRIG3_TRIGGER_IN_WIDTH   =>     C_TRIG3_TRIGGER_IN_WIDTH,
    C_TRIG3_UNIT_COUNTER_WIDTH =>     C_TRIG3_UNIT_COUNTER_WIDTH,
    C_TRIG3_UNIT_MATCH_TYPE    =>     C_TRIG3_UNIT_MATCH_TYPE,

    C_TRIG4_UNITS              =>     C_TRIG4_UNITS,
    C_TRIG4_TRIGGER_IN_WIDTH   =>     C_TRIG4_TRIGGER_IN_WIDTH,
    C_TRIG4_UNIT_COUNTER_WIDTH =>     C_TRIG4_UNIT_COUNTER_WIDTH,
    C_TRIG4_UNIT_MATCH_TYPE    =>     C_TRIG4_UNIT_MATCH_TYPE,

    C_TRIG5_UNITS              =>     C_TRIG5_UNITS,
    C_TRIG5_TRIGGER_IN_WIDTH   =>     C_TRIG5_TRIGGER_IN_WIDTH,
    C_TRIG5_UNIT_COUNTER_WIDTH =>     C_TRIG5_UNIT_COUNTER_WIDTH,
    C_TRIG5_UNIT_MATCH_TYPE    =>     C_TRIG5_UNIT_MATCH_TYPE,

    C_TRIG6_UNITS              =>     C_TRIG6_UNITS,
    C_TRIG6_TRIGGER_IN_WIDTH   =>     C_TRIG6_TRIGGER_IN_WIDTH,
    C_TRIG6_UNIT_COUNTER_WIDTH =>     C_TRIG6_UNIT_COUNTER_WIDTH,
    C_TRIG6_UNIT_MATCH_TYPE    =>     C_TRIG6_UNIT_MATCH_TYPE,

    C_TRIG7_UNITS              =>     C_TRIG7_UNITS,
    C_TRIG7_TRIGGER_IN_WIDTH   =>     C_TRIG7_TRIGGER_IN_WIDTH,
    C_TRIG7_UNIT_COUNTER_WIDTH =>     C_TRIG7_UNIT_COUNTER_WIDTH,
    C_TRIG7_UNIT_MATCH_TYPE    =>     C_TRIG7_UNIT_MATCH_TYPE,

    C_TRIG8_UNITS              =>     C_TRIG8_UNITS,
    C_TRIG8_TRIGGER_IN_WIDTH   =>     C_TRIG8_TRIGGER_IN_WIDTH,
    C_TRIG8_UNIT_COUNTER_WIDTH =>     C_TRIG8_UNIT_COUNTER_WIDTH,
    C_TRIG8_UNIT_MATCH_TYPE    =>     C_TRIG8_UNIT_MATCH_TYPE,

    C_TRIG9_UNITS              =>     C_TRIG9_UNITS,
    C_TRIG9_TRIGGER_IN_WIDTH   =>     C_TRIG9_TRIGGER_IN_WIDTH,
    C_TRIG9_UNIT_COUNTER_WIDTH =>     C_TRIG9_UNIT_COUNTER_WIDTH,
    C_TRIG9_UNIT_MATCH_TYPE    =>     C_TRIG9_UNIT_MATCH_TYPE,

    C_TRIG10_UNITS =>     C_TRIG10_UNITS,
    C_TRIG10_TRIGGER_IN_WIDTH =>     C_TRIG10_TRIGGER_IN_WIDTH,
    C_TRIG10_UNIT_COUNTER_WIDTH =>     C_TRIG10_UNIT_COUNTER_WIDTH,
    C_TRIG10_UNIT_MATCH_TYPE =>     C_TRIG10_UNIT_MATCH_TYPE,

    C_TRIG11_UNITS =>     C_TRIG11_UNITS,
    C_TRIG11_TRIGGER_IN_WIDTH =>     C_TRIG11_TRIGGER_IN_WIDTH,
    C_TRIG11_UNIT_COUNTER_WIDTH =>     C_TRIG11_UNIT_COUNTER_WIDTH,
    C_TRIG11_UNIT_MATCH_TYPE =>     C_TRIG11_UNIT_MATCH_TYPE,

    C_TRIG12_UNITS =>     C_TRIG12_UNITS,
    C_TRIG12_TRIGGER_IN_WIDTH =>     C_TRIG12_TRIGGER_IN_WIDTH,
    C_TRIG12_UNIT_COUNTER_WIDTH =>     C_TRIG12_UNIT_COUNTER_WIDTH,
    C_TRIG12_UNIT_MATCH_TYPE =>     C_TRIG12_UNIT_MATCH_TYPE,

    C_TRIG13_UNITS =>     C_TRIG13_UNITS,
    C_TRIG13_TRIGGER_IN_WIDTH =>     C_TRIG13_TRIGGER_IN_WIDTH,
    C_TRIG13_UNIT_COUNTER_WIDTH =>     C_TRIG13_UNIT_COUNTER_WIDTH,
    C_TRIG13_UNIT_MATCH_TYPE =>     C_TRIG13_UNIT_MATCH_TYPE,

    C_TRIG14_UNITS =>     C_TRIG14_UNITS,
    C_TRIG14_TRIGGER_IN_WIDTH =>     C_TRIG14_TRIGGER_IN_WIDTH,
    C_TRIG14_UNIT_COUNTER_WIDTH =>     C_TRIG14_UNIT_COUNTER_WIDTH,
    C_TRIG14_UNIT_MATCH_TYPE =>     C_TRIG14_UNIT_MATCH_TYPE,

    C_TRIG15_UNITS =>     C_TRIG15_UNITS,
    C_TRIG15_TRIGGER_IN_WIDTH =>     C_TRIG15_TRIGGER_IN_WIDTH,
    C_TRIG15_UNIT_COUNTER_WIDTH =>     C_TRIG15_UNIT_COUNTER_WIDTH,
    C_TRIG15_UNIT_MATCH_TYPE =>     C_TRIG15_UNIT_MATCH_TYPE
    )
    port map
    (
"
		if { $params(C_ENABLE_TRIGGER_OUT) != 0 } {
	    puts $hdlfile "      trig_out    => trig_out,"
		}
		if { $params(C_TRIG0_UNITS) > 0 } {
	    puts $hdlfile "      trig0       => trig0,"
		}
		if { $params(C_TRIG1_UNITS) > 0 } {
	    puts $hdlfile "      trig1       => trig1,"
		}
		if { $params(C_TRIG2_UNITS) > 0 } {
	    puts $hdlfile "      trig2       => trig2,"
		}
		if { $params(C_TRIG3_UNITS) > 0 } {
	    puts $hdlfile "      trig3       => trig3,"
		}
		if { $params(C_TRIG4_UNITS) > 0 } {
	    puts $hdlfile "      trig4       => trig4,"
		}
		if { $params(C_TRIG5_UNITS) > 0 } {
	    puts $hdlfile "      trig5       => trig5,"
		}
		if { $params(C_TRIG6_UNITS) > 0 } {
	    puts $hdlfile "      trig6       => trig6,"
		}
		if { $params(C_TRIG7_UNITS) > 0 } {
	    puts $hdlfile "      trig7       => trig7,"
		}
		if { $params(C_TRIG8_UNITS) > 0 } {
	    puts $hdlfile "      trig8       => trig8,"
		}
		if { $params(C_TRIG9_UNITS) > 0 } {
	    puts $hdlfile "      trig9       => trig9,"
		}
		if { $params(C_TRIG10_UNITS) > 0 } {
	    puts $hdlfile "      trig10      => trig10,"
		}
		if { $params(C_TRIG11_UNITS) > 0 } {
	    puts $hdlfile "      trig11      => trig11,"
		}
		if { $params(C_TRIG12_UNITS) > 0 } {
	    puts $hdlfile "      trig12      => trig12,"
		}
		if { $params(C_TRIG13_UNITS) > 0 } {
	    puts $hdlfile "      trig13      => trig13,"
		}
		if { $params(C_TRIG14_UNITS) > 0 } {
	    puts $hdlfile "      trig14      => trig14,"
		}
		if { $params(C_TRIG15_UNITS) > 0 } {
	    puts $hdlfile "      trig15      => trig15,"
		}
		if { $params(C_DATA_SAME_AS_TRIGGER) == 0 } {
	    puts $hdlfile "      data        => data,"
		}
		puts $hdlfile "
      clk           => clk,
      control       => chipscope_ila_control
    );

end architecture imp;
"
	}
	
	close $hdlfile
	
}

proc update_cdc_file {handle param_table} {
	
	# put the PARAMETER name value pairs into an associative array 
	# for easy access Ex: set instance_name $params(INSTANCE)
	array set params $param_table
	
	# read the CDC file
	set cdc_name "$params(INSTANCE).cdc"
	set cdc_path "implementation/$params(INSTANCE)_wrapper/$cdc_name"
	set cdc_abspath "[file nativename [pwd]]/$cdc_path"
	set cdc_file [open $cdc_abspath "r"]
	set data [read $cdc_file]
	close $cdc_file
	set cdc_lines [split $data \n]
	
	# get all signal names connected to each port and replace
	# the original signal names in the CDC file
	set mhs_handle [xget_hw_parent_handle $handle]
	set signals [xget_hw_port_handle $handle *]
	set busindex 1
	set buslines {}
	foreach item $signals {
		regsub -all { } [xget_hw_value $item] {} value
		if {$value != ""} {
			set portname [xget_hw_name $item]
                        if {[regexp -nocase {TRIG[0-9][0-9]*} $portname]} {
				set signals [split $value "&"]
				set signalwidth $params(C_${portname}_TRIGGER_IN_WIDTH)
				
				# Get all source names and create unique names by appending a numeric suffix
				set sourcenameList {}
				foreach signal $signals {
					set source [xget_connected_ports_handle $mhs_handle $signal "SOURCE"]
					if {$source == ""} {
						continue ;# Unable to evaluate this signal - skip it
					}
					set sourcename [xget_hw_name $source]
					lappend sourcenameList $sourcename
				}
				foreach sourcename $sourcenameList {
					set sourcenameIndices [lsearch -exact -all $sourcenameList "$sourcename"]
					if {[llength $sourcenameIndices] > 1} {
						set suffix 0
						foreach sourcenameIndex $sourcenameIndices {
							set newname [lindex $sourcenameList $sourcenameIndex]_$suffix
							lset sourcenameList $sourcenameIndex $newname
							incr suffix
						}
					}
				}
				
				# Get connected ports and their ranges and create signal array
				set signalindex [expr $signalwidth - 1]
				# puts "DEBUG 1: $portname - $signalindex"
				set sourcenameIndex 0
				foreach signal $signals {
					set source [xget_connected_ports_handle $mhs_handle $signal "SOURCE"]
					if {$source == ""} {
						continue ;# Unable to evaluate this signal - skip it
					}
					set vec [xget_hw_subproperty_value $source "VEC"]
					set sourcename [lindex $sourcenameList $sourcenameIndex]
					incr sourcenameIndex
					if {$vec == ""} {
						set first 0
						set last  0
					} else {
						if {[regexp {\[(.*)\:(.*)\]} $vec match firstStr lastStr]} {
							# Evaluate index expressions
							set parent [xget_hw_parent_handle $source]
							set signalParams [xget_hw_parameter_handle $parent *]
							foreach param $signalParams {
								set paramName  [xget_hw_name  $param]
								set paramValue [xget_hw_value $param]
								set $paramName $paramValue
							}
							regsub -all {[^A-Za-z0-9_]} $firstStr { & } firstStr
							regsub -all {[A-Za-z][A-Za-z0-9_]+} $firstStr {$&} firstStr
							if {[catch {set first [eval expr [split $firstStr]]} err]} {
								continue ;# Unable to evaluate this signal - skip it
							}
							regsub -all {[^A-Za-z0-9_]} $lastStr { & } lastStr
							regsub -all {[A-Za-z][A-Za-z0-9_]+} $lastStr {$&} lastStr
							if {[catch {set last [eval expr [split $lastStr]]} err]} {
								continue ;# Unable to evaluate this signal - skip it
							}
						} else {
							continue ;# Unable to evaluate this signal - skip it
						}
					}
					# puts "  DEBUG 2: $sourcename - $first $last"
					
					set buschannels ""
					if {$first == $last} {
						set signalarray($signalindex) "${sourcename}"
						set signalindex [expr $signalindex - 1]
					} elseif {$first <= $last} {
						set index $first
						while {$index <= $last} {
							set signalarray($signalindex) "${sourcename}\[$index\]"
							set buschannels "$signalindex $buschannels"
							incr index
							set signalindex [expr $signalindex - 1]
						}
					} else {
						set index $first
						while {$index >= $last} {
							set signalarray($signalindex) "${sourcename}\[$index\]"
							set buschannels "$buschannels $signalindex"
							set index       [expr $index - 1]
							set signalindex [expr $signalindex - 1]
						}
					}
					
					# Create bus declarations
					if {$first != $last} {
						set busindextext [format "%04d" $busindex]
						lappend buslines "SignalExport.bus<${busindextext}>.channelList=${buschannels}"
						lappend buslines "SignalExport.bus<${busindextext}>.name=${sourcename}"
						lappend buslines "SignalExport.bus<${busindextext}>.offset=0.0"
						lappend buslines "SignalExport.bus<${busindextext}>.precision=0"
						lappend buslines "SignalExport.bus<${busindextext}>.radix=Hex"
						lappend buslines "SignalExport.bus<${busindextext}>.scaleFactor=1.0"
						incr busindex
					}
				}
				
				# Replace original signal names in the CDC file
				set signalindex 0
				while {$signalindex < $signalwidth} {
					set lineindex 0
					foreach line $cdc_lines {
						set portindex {\[([0-9]+)\]}
						set port "${portname}${portindex}"
						if {[regexp $port $line match index] && 
								[array names signalarray $index] != ""} {
							regsub $port $line $signalarray($index) changedline
							set cdc_lines [lreplace $cdc_lines $lineindex $lineindex "$changedline"]
							# puts "  DEBUG 3: $line - $changedline"
							break
						}
						incr lineindex
					}
					incr signalindex
				}
			}
		}
	}
	
	# Insert bus declarations in the CDC file after existing bus declarations
	set lineindex   0
	set insertindex 2
	foreach line $cdc_lines {
		incr lineindex
		if {[regexp "SignalExport\.bus<0000>" $line]} {
			set insertindex $lineindex
		}
	}
	foreach line $buslines {
		set cdc_lines [linsert $cdc_lines $insertindex $line]
		incr insertindex
	}
	
	# Write the updated CDC file
#	set cdc_abspath "[file nativename [pwd]]/$cdc_name"
	set cdc_file [open $cdc_abspath "w"]
	foreach line $cdc_lines {
		puts $cdc_file $line
	}
	close $cdc_file
}
