module rwcontrol (
                   //out
				   pslverr,
				   prdata,
				   mode_detect_nmi,
				   int_mode,
				   iscra,
				   iscrb,
				   irq,
				   ipra,
				   iprb,
				   iprc,
				   iprd,
				   ipre,
				   iprf,				   
				   //sys
				   pclk,
				   preset,
				   //in
				   pwrite,
				   psel,
				   penable,
				   paddr,
				   pwdata,
				   detect_ext_int
				 );
				 
`include "address.v"

/*
*
*
*/
output pslverr;
output[31:0]prdata;
output[1:0]mode_detect_nmi;
output[7:0]iscra;
output[7:0]iscrb;
output[7:0]irq;
output[7:0]ipra;
output[7:0]iprb;
output[7:0]iprc;
output[7:0]iprd;
output[7:0]ipre;
output[7:0]iprf;
output int_mode;
/*
*
*
*/
input pclk;
input preset;
input pwrite;
input psel;
input penable;
input[31:0]paddr;
input[31:0]pwdata;
input[7:0]detect_ext_int;
/*
*
*
*/
reg[31:0]prdata;
reg[7:0]sysclr;
reg[7:0]iscra;
reg[7:0]iscrb;
reg[7:0]ipra;
reg[7:0]iprb;
reg[7:0]iprc;
reg[7:0]iprd;
reg[7:0]ipre;
reg[7:0]iprf;
reg[7:0]isr;
reg[7:0]ier;
reg[15:0]sel;
wire[1:0]mode_detect_nmi;
wire pslverr;
wire int_mode;
wire[7:0]irq;
/*
*
*
*/

/****** decode address ********/
always @(paddr or psel or pwrite)
  begin
    if( (pwrite && psel) || ( (!pwrite) && psel) )
	  begin
	    case(paddr)
		`sysclr:sel[0]  <= 1'b1;
		`ier   :sel[1]  <= 1'b1;
		`isr   :sel[2]  <= 1'b1;
		`iscra :sel[3]  <= 1'b1;
		`iscrb :sel[4]  <= 1'b1;
		`ipra  :sel[5]  <= 1'b1;
		`iprb  :sel[6]  <= 1'b1;
		`iprc  :sel[7]  <= 1'b1;
		`iprd  :sel[8]  <= 1'b1;
		`ipre   :sel[9]  <= 1'b1;
		`iprf   :sel[10] <= 1'b1;
		32'h7940000b   :sel[11] <= 1'b1;
		32'h7940000c   :sel[12] <= 1'b1;
		32'h7940000d   :sel[13] <= 1'b1;
		32'h7940000e   :sel[14] <= 1'b1;
		32'h7940000f   :sel[15] <= 1'b1;
		default:sel <= 16'h0000;
		endcase
	  end
    else
	  sel <= 16'h0000;
  end
/*
*
*
*/

/******** system control register *********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  sysclr <= 8'h00;
	else if(pwrite && psel && penable && sel[0])
	  sysclr <= pwdata;
	else
	  sysclr <= sysclr;
  end
/*
*
*
*/
/******* interrupt enable register ********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  ier <= 8'h00;
	else if(pwrite && psel && penable && sel[1])
	  ier <= pwdata;
	else
	  ier <= ier;
  end
/*
*
*
*/
/********* interrupt status register *******/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  isr <= 8'h00;
	else if(pwrite && psel && penable && sel[2])
	  isr <= isr & pwdata;
	else
	  isr <= isr | detect_ext_int;
  end  
/*
*
*
*/
/********* interrupt sense control regiter A *********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  iscra <= 8'h00;
	else if(pwrite && psel && penable && sel[3])
	  iscra <= pwdata;
	else
	  iscra <= iscra;
  end
/*
*
*
*/
/********** interrupt sense control register B *********/  
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  iscrb <= 8'h00;
	else if(pwrite && psel && penable && sel[4])
	  iscrb <= pwdata;
	else
	  iscrb <= iscrb;
  end
/*
*
*
*/
/********** interrupt priority register A **********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  ipra <= 8'h00;
	else if(pwrite && psel && penable && sel[5])
	  ipra <= pwdata;
	else
	  ipra <= ipra;
  end  
/*
*
*
*/
/********** interrupt priority register B **********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  iprb <= 8'h00;
	else if(pwrite && psel && penable && sel[6])
	  iprb <= pwdata;
	else
	  iprb <= iprb;
  end 
/*
*
*
*/
/********** interrupt priority register C **********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  iprc <= 8'h00;
	else if(pwrite && psel && penable && sel[7])
	  iprc <= pwdata;
	else
	  iprc <= iprc;
  end  
/*
*
*
*/
/********** interrupt priority register D **********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  iprd <= 8'h00;
	else if(pwrite && psel && penable && sel[8])
	  iprd <= pwdata;
	else
	  iprd <= iprd;
  end   
/*
*
*
*/
/********** interrupt priority register E **********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  ipre <= 8'h00;
	else if(pwrite && psel && penable && sel[9])
	  ipre <= pwdata;
	else
	  ipre <= ipre;
  end   
/*
*
*
*/
/********** interrupt priority register F **********/
always @(posedge pclk or negedge preset)
  begin
    if(!preset)
	  iprf <= 8'h00;
	else if(pwrite && psel && penable && sel[10])
	  iprf <= pwdata;
	else
	  iprf <= iprf;
  end 
/*
*
*
*/
/*********** read data from register **************/
always @(*)
  begin
    if( (!pwrite) && psel && penable )
	    case(paddr)
		`sysclr:prdata <= { 24'h000000,sysclr};
	    `ier   :prdata <= { 24'h000000,ier};
		`isr   :prdata <= { 24'h000000,isr};
		`iscra :prdata <= { 24'h000000,iscra};
		`iscrb :prdata <= { 24'h000000,iscrb};
		`ipra  :prdata <= { 24'h000000,ipra};
		`iprb  :prdata <= { 24'h000000,iprb};
		`iprc  :prdata <= { 24'h000000,iprc};
		`iprd  :prdata <= { 24'h000000,iprd};
		`ipre  :prdata <= { 24'h000000,ipre};
		`iprf  :prdata <= { 24'h000000,iprf};	
		default:prdata <= 32'h00000000;
		endcase
	else
	  prdata <= prdata;
  end
/*
*
*
*/
assign mode_detect_nmi = sysclr[1:0];
assign pslverr = sel[11] | sel[12] | sel[13] | sel[14] | sel[15];
assign irq = isr & ier;
assign int_mode=sysclr[2];


endmodule

  
  

 
				 

				   